import mock
import os
import pandas as pd
from datetime import datetime
from flexmock import flexmock
from sportsreference import utils
from sportsreference.constants import HOME
from sportsreference.ncaab.constants import BOXSCORES_URL, SCHEDULE_URL
from sportsreference.ncaab.boxscore import Boxscore, Boxscores


MONTH = 11
YEAR = 2017

BOXSCORE = '2017-11-24-21-purdue'


def read_file(filename):
    filepath = os.path.join(os.path.dirname(__file__), 'ncaab', filename)
    return open('%s' % filepath, 'r').read()


def mock_pyquery(url):
    class MockPQ:
        def __init__(self, html_contents):
            self.status_code = 200
            self.html_contents = html_contents
            self.text = html_contents

        def __call__(self, div):
            return read_file('table.html')

    if url == BOXSCORES_URL % (MONTH, 11, YEAR):
        return MockPQ(read_file('boxscores-11-11-2017.html'))
    if url == BOXSCORES_URL % (MONTH, 11, YEAR):
        return MockPQ(read_file('boxscores-11-12-2017.html'))
    boxscore = read_file('%s.html' % BOXSCORE)
    return MockPQ(boxscore)


class MockDateTime:
    def __init__(self, year, month):
        self.year = year
        self.month = month


class TestNCAABBoxscore:
    @mock.patch('requests.get', side_effect=mock_pyquery)
    def setup_method(self, *args, **kwargs):
        self.results = {
            'date': 'November 24, 2017',
            'location': 'Imperial Arena at Atlantis Resort, Paradise Island',
            'winner': HOME,
            'winning_name': 'Purdue',
            'winning_abbr': 'PURDUE',
            'losing_name': 'Arizona',
            'losing_abbr': 'ARIZONA',
            'pace': 71.8,
            'away_ranking': 2,
            'away_win_percentage': .500,
            'away_wins': 3,
            'away_losses': 3,
            'away_minutes_played': 200,
            'away_field_goals': 25,
            'away_field_goal_attempts': 59,
            'away_field_goal_percentage': .424,
            'away_two_point_field_goals': 22,
            'away_two_point_field_goal_attempts': 42,
            'away_two_point_field_goal_percentage': .524,
            'away_three_point_field_goals': 3,
            'away_three_point_field_goal_attempts': 17,
            'away_three_point_field_goal_percentage': .176,
            'away_free_throws': 11,
            'away_free_throw_attempts': 16,
            'away_free_throw_percentage': .688,
            'away_offensive_rebounds': 6,
            'away_defensive_rebounds': 16,
            'away_total_rebounds': 22,
            'away_assists': 13,
            'away_steals': 4,
            'away_blocks': 3,
            'away_turnovers': 11,
            'away_personal_fouls': 20,
            'away_points': 64,
            'away_true_shooting_percentage': .480,
            'away_effective_field_goal_percentage': .449,
            'away_three_point_attempt_rate': .288,
            'away_free_throw_attempt_rate': .271,
            'away_offensive_rebound_percentage': 18.8,
            'away_defensive_rebound_percentage': 72.7,
            'away_total_rebound_percentage': 40.7,
            'away_assist_percentage': 52.0,
            'away_steal_percentage': 5.6,
            'away_block_percentage': 9.4,
            'away_turnover_percentage': 14.3,
            'away_offensive_rating': 88.9,
            'away_defensive_rating': 123.6,
            'home_ranking': 18,
            'home_win_percentage': .714,
            'home_wins': 5,
            'home_losses': 2,
            'home_minutes_played': 200,
            'home_field_goals': 31,
            'home_field_goal_attempts': 54,
            'home_field_goal_percentage': .574,
            'home_two_point_field_goals': 20,
            'home_two_point_field_goal_attempts': 32,
            'home_two_point_field_goal_percentage': .625,
            'home_three_point_field_goals': 11,
            'home_three_point_field_goal_attempts': 22,
            'home_three_point_field_goal_percentage': .500,
            'home_free_throws': 16,
            'home_free_throw_attempts': 21,
            'home_free_throw_percentage': .762,
            'home_offensive_rebounds': 6,
            'home_defensive_rebounds': 26,
            'home_total_rebounds': 32,
            'home_assists': 19,
            'home_steals': 6,
            'home_blocks': 7,
            'home_turnovers': 14,
            'home_personal_fouls': 19,
            'home_points': 89,
            'home_true_shooting_percentage': .696,
            'home_effective_field_goal_percentage': .676,
            'home_three_point_attempt_rate': .407,
            'home_free_throw_attempt_rate': .389,
            'home_offensive_rebound_percentage': 27.3,
            'home_defensive_rebound_percentage': 81.3,
            'home_total_rebound_percentage': 59.3,
            'home_assist_percentage': 61.3,
            'home_steal_percentage': 8.3,
            'home_block_percentage': 16.7,
            'home_turnover_percentage': 18.1,
            'home_offensive_rating': 123.6,
            'home_defensive_rating': 88.9
        }
        flexmock(utils) \
            .should_receive('_todays_date') \
            .and_return(MockDateTime(YEAR, MONTH))

        self.boxscore = Boxscore('2017-11-24-21-purdue')

    def test_ncaab_boxscore_returns_requested_boxscore(self):
        for attribute, value in self.results.items():
            assert getattr(self.boxscore, attribute) == value

    def test_invalid_url_yields_empty_class(self):
        flexmock(Boxscore) \
            .should_receive('_retrieve_html_page') \
            .and_return(None)

        boxscore = Boxscore(BOXSCORE)

        for key, value in boxscore.__dict__.items():
            if key == '_uri':
                continue
            assert value is None

    def test_ncaab_boxscore_dataframe_returns_dataframe_of_all_values(self):
        df = pd.DataFrame([self.results], index=[BOXSCORE])

        # Pandas doesn't natively allow comparisons of DataFrames.
        # Concatenating the two DataFrames (the one generated during the test
        # and the expected one above) and dropping duplicate rows leaves only
        # the rows that are unique between the two frames. This allows a quick
        # check of the DataFrame to see if it is empty - if so, all rows are
        # duplicates, and they are equal.
        frames = [df, self.boxscore.dataframe]
        df1 = pd.concat(frames).drop_duplicates(keep=False)

        assert df1.empty

    def test_ncaab_boxscore_players(self):
        boxscore = Boxscore(BOXSCORE)

        assert len(boxscore.home_players) == 12
        assert len(boxscore.away_players) == 10

        for player in boxscore.home_players:
            assert not player.dataframe.empty
        for player in boxscore.away_players:
            assert not player.dataframe.empty


class TestNCAABBoxscores:
    def setup_method(self):
        self.expected = {
            '11-11-2017': [
                {'boxscore': '2017-11-11-13-buffalo',
                 'away_name': 'Canisius',
                 'away_abbr': 'canisius',
                 'away_score': 75,
                 'away_rank': None,
                 'home_name': 'Buffalo',
                 'home_abbr': 'buffalo',
                 'home_score': 80,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Buffalo',
                 'winning_abbr': 'buffalo',
                 'losing_name': 'Canisius',
                 'losing_abbr': 'canisius'},
                {'boxscore': '2017-11-11-13-fairfield',
                 'away_name': 'Penn',
                 'away_abbr': 'pennsylvania',
                 'away_score': 72,
                 'away_rank': None,
                 'home_name': 'Fairfield',
                 'home_abbr': 'fairfield',
                 'home_score': 80,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Fairfield',
                 'winning_abbr': 'fairfield',
                 'losing_name': 'Penn',
                 'losing_abbr': 'pennsylvania'},
                {'boxscore': '2017-11-11-13-florida-gulf-coast',
                 'away_name': 'Illinois State',
                 'away_abbr': 'illinois-state',
                 'away_score': 87,
                 'away_rank': None,
                 'home_name': 'Florida Gulf Coast',
                 'home_abbr': 'florida-gulf-coast',
                 'home_score': 98,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Florida Gulf Coast',
                 'winning_abbr': 'florida-gulf-coast',
                 'losing_name': 'Illinois State',
                 'losing_abbr': 'illinois-state'},
                {'boxscore': '2017-11-11-14-bradley',
                 'away_name': 'IUPUI',
                 'away_abbr': 'iupui',
                 'away_score': 53,
                 'away_rank': None,
                 'home_name': 'Bradley',
                 'home_abbr': 'bradley',
                 'home_score': 68,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Bradley',
                 'winning_abbr': 'bradley',
                 'losing_name': 'IUPUI',
                 'losing_abbr': 'iupui'},
                {'boxscore': '2017-11-11-14-ohio',
                 'away_name': 'Alabama A&M',
                 'away_abbr': 'alabama-am',
                 'away_score': 53,
                 'away_rank': None,
                 'home_name': 'Ohio',
                 'home_abbr': 'ohio',
                 'home_score': 61,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Ohio',
                 'winning_abbr': 'ohio',
                 'losing_name': 'Alabama A&M',
                 'losing_abbr': 'alabama-am'},
                {'boxscore': '2017-11-11-14-quinnipiac',
                 'away_name': 'Dartmouth',
                 'away_abbr': 'dartmouth',
                 'away_score': 77,
                 'away_rank': None,
                 'home_name': 'Quinnipiac',
                 'home_abbr': 'quinnipiac',
                 'home_score': 78,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Quinnipiac',
                 'winning_abbr': 'quinnipiac',
                 'losing_name': 'Dartmouth',
                 'losing_abbr': 'dartmouth'},
                {'boxscore': '2017-11-11-14-western-michigan',
                 'away_name': 'Siena Heights',
                 'away_abbr': 'Siena Heights',
                 'away_score': 72,
                 'away_rank': None,
                 'home_name': 'Western Michigan',
                 'home_abbr': 'western-michigan',
                 'home_score': 103,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Western Michigan',
                 'winning_abbr': 'western-michigan',
                 'losing_name': 'Siena Heights',
                 'losing_abbr': 'Siena Heights'},
                {'boxscore': '2017-11-11-15-appalachian-state',
                 'away_name': 'Toccoa Falls',
                 'away_abbr': 'Toccoa Falls',
                 'away_score': 34,
                 'away_rank': None,
                 'home_name': 'Appalachian State',
                 'home_abbr': 'appalachian-state',
                 'home_score': 135,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Appalachian State',
                 'winning_abbr': 'appalachian-state',
                 'losing_name': 'Toccoa Falls',
                 'losing_abbr': 'Toccoa Falls'},
                {'boxscore': '2017-11-11-15-brown',
                 'away_name': 'Johnson & Wales (RI)',
                 'away_abbr': 'Johnson & Wales (RI)',
                 'away_score': 78,
                 'away_rank': None,
                 'home_name': 'Brown',
                 'home_abbr': 'brown',
                 'home_score': 106,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Brown',
                 'winning_abbr': 'brown',
                 'losing_name': 'Johnson & Wales (RI)',
                 'losing_abbr': 'Johnson & Wales (RI)'},
                {'boxscore': '2017-11-11-15-drake',
                 'away_name': 'Coe',
                 'away_abbr': 'Coe',
                 'away_score': 54,
                 'away_rank': None,
                 'home_name': 'Drake',
                 'home_abbr': 'drake',
                 'home_score': 101,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Drake',
                 'winning_abbr': 'drake',
                 'losing_name': 'Coe',
                 'losing_abbr': 'Coe'},
                {'boxscore': '2017-11-11-15-la-salle',
                 'away_name': "St. Peter's",
                 'away_abbr': 'saint-peters',
                 'away_score': 40,
                 'away_rank': None,
                 'home_name': 'La Salle',
                 'home_abbr': 'la-salle',
                 'home_score': 61,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'La Salle',
                 'winning_abbr': 'la-salle',
                 'losing_name': "St. Peter's",
                 'losing_abbr': 'saint-peters'},
                {'boxscore': '2017-11-11-15-longwood',
                 'away_name': 'Stephen F. Austin',
                 'away_abbr': 'stephen-f-austin',
                 'away_score': 74,
                 'away_rank': None,
                 'home_name': 'Longwood',
                 'home_abbr': 'longwood',
                 'home_score': 61,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Stephen F. Austin',
                 'winning_abbr': 'stephen-f-austin',
                 'losing_name': 'Longwood',
                 'losing_abbr': 'longwood'},
                {'boxscore': '2017-11-11-16-arkansas-little-rock',
                 'away_name': 'Ouachita',
                 'away_abbr': 'Ouachita',
                 'away_score': 81,
                 'away_rank': None,
                 'home_name': 'Little Rock',
                 'home_abbr': 'arkansas-little-rock',
                 'home_score': 79,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Ouachita',
                 'winning_abbr': 'Ouachita',
                 'losing_name': 'Little Rock',
                 'losing_abbr': 'arkansas-little-rock'},
                {'boxscore': '2017-11-11-16-california-santa-barbara',
                 'away_name': 'North Dakota State',
                 'away_abbr': 'north-dakota-state',
                 'away_score': 66,
                 'away_rank': None,
                 'home_name': 'UCSB',
                 'home_abbr': 'california-santa-barbara',
                 'home_score': 85,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'UCSB',
                 'winning_abbr': 'california-santa-barbara',
                 'losing_name': 'North Dakota State',
                 'losing_abbr': 'north-dakota-state'},
                {'boxscore': '2017-11-11-16-depaul',
                 'away_name': 'Notre Dame',
                 'away_abbr': 'notre-dame',
                 'away_score': 72,
                 'away_rank': 14,
                 'home_name': 'DePaul',
                 'home_abbr': 'depaul',
                 'home_score': 58,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Notre Dame',
                 'winning_abbr': 'notre-dame',
                 'losing_name': 'DePaul',
                 'losing_abbr': 'depaul'},
                {'boxscore': '2017-11-11-17-duquesne',
                 'away_name': 'St. Francis (NY)',
                 'away_abbr': 'st-francis-ny',
                 'away_score': 70,
                 'away_rank': None,
                 'home_name': 'Duquesne',
                 'home_abbr': 'duquesne',
                 'home_score': 80,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Duquesne',
                 'winning_abbr': 'duquesne',
                 'losing_name': 'St. Francis (NY)',
                 'losing_abbr': 'st-francis-ny'},
                {'boxscore': '2017-11-11-18-njit',
                 'away_name': 'Wagner',
                 'away_abbr': 'wagner',
                 'away_score': 60,
                 'away_rank': None,
                 'home_name': 'NJIT',
                 'home_abbr': 'njit',
                 'home_score': 49,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Wagner',
                 'winning_abbr': 'wagner',
                 'losing_name': 'NJIT',
                 'losing_abbr': 'njit'},
                {'boxscore': '2017-11-11-19-akron',
                 'away_name': 'Cleveland State',
                 'away_abbr': 'cleveland-state',
                 'away_score': 57,
                 'away_rank': None,
                 'home_name': 'Akron',
                 'home_abbr': 'akron',
                 'home_score': 67,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Akron',
                 'winning_abbr': 'akron',
                 'losing_name': 'Cleveland State',
                 'losing_abbr': 'cleveland-state'},
                {'boxscore': '2017-11-11-19-duke',
                 'away_name': 'Utah Valley',
                 'away_abbr': 'utah-valley',
                 'away_score': 69,
                 'away_rank': None,
                 'home_name': 'Duke',
                 'home_abbr': 'duke',
                 'home_score': 99,
                 'home_rank': 1,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Duke',
                 'winning_abbr': 'duke',
                 'losing_name': 'Utah Valley',
                 'losing_abbr': 'utah-valley'},
                {'boxscore': '2017-11-11-19-elon',
                 'away_name': 'Peace',
                 'away_abbr': 'Peace',
                 'away_score': 53,
                 'away_rank': None,
                 'home_name': 'Elon',
                 'home_abbr': 'elon',
                 'home_score': 96,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Elon',
                 'winning_abbr': 'elon',
                 'losing_name': 'Peace',
                 'losing_abbr': 'Peace'},
                {'boxscore': '2017-11-11-19-marist',
                 'away_name': 'Lehigh',
                 'away_abbr': 'lehigh',
                 'away_score': 84,
                 'away_rank': None,
                 'home_name': 'Marist',
                 'home_abbr': 'marist',
                 'home_score': 76,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Lehigh',
                 'winning_abbr': 'lehigh',
                 'losing_name': 'Marist',
                 'losing_abbr': 'marist'},
                {'boxscore': '2017-11-11-19-michigan',
                 'away_name': 'North Florida',
                 'away_abbr': 'north-florida',
                 'away_score': 66,
                 'away_rank': None,
                 'home_name': 'Michigan',
                 'home_abbr': 'michigan',
                 'home_score': 86,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Michigan',
                 'winning_abbr': 'michigan',
                 'losing_name': 'North Florida',
                 'losing_abbr': 'north-florida'},
                {'boxscore': '2017-11-11-19-north-carolina-wilmington',
                 'away_name': 'North Carolina Wesleyan',
                 'away_abbr': 'North Carolina Wesleyan',
                 'away_score': 81,
                 'away_rank': None,
                 'home_name': 'North Carolina-Wilmington',
                 'home_abbr': 'north-carolina-wilmington',
                 'home_score': 105,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'North Carolina-Wilmington',
                 'winning_abbr': 'north-carolina-wilmington',
                 'losing_name': 'North Carolina Wesleyan',
                 'losing_abbr': 'North Carolina Wesleyan'},
                {'boxscore': '2017-11-11-19-presbyterian',
                 'away_name': 'Johnson University',
                 'away_abbr': 'Johnson University',
                 'away_score': 87,
                 'away_rank': None,
                 'home_name': 'Presbyterian',
                 'home_abbr': 'presbyterian',
                 'home_score': 107,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Presbyterian',
                 'winning_abbr': 'presbyterian',
                 'losing_name': 'Johnson University',
                 'losing_abbr': 'Johnson University'},
                {'boxscore': '2017-11-11-19-sam-houston-state',
                 'away_name': 'Texas-Tyler',
                 'away_abbr': 'Texas-Tyler',
                 'away_score': 51,
                 'away_rank': None,
                 'home_name': 'Sam Houston State',
                 'home_abbr': 'sam-houston-state',
                 'home_score': 91,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Sam Houston State',
                 'winning_abbr': 'sam-houston-state',
                 'losing_name': 'Texas-Tyler',
                 'losing_abbr': 'Texas-Tyler'},
                {'boxscore': '2017-11-11-19-south-dakota',
                 'away_name': 'Mayville State',
                 'away_abbr': 'Mayville State',
                 'away_score': 59,
                 'away_rank': None,
                 'home_name': 'South Dakota',
                 'home_abbr': 'south-dakota',
                 'home_score': 87,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'South Dakota',
                 'winning_abbr': 'south-dakota',
                 'losing_name': 'Mayville State',
                 'losing_abbr': 'Mayville State'},
                {'boxscore': '2017-11-11-19-toledo',
                 'away_name': "St. Joseph's",
                 'away_abbr': 'saint-josephs',
                 'away_score': 87,
                 'away_rank': None,
                 'home_name': 'Toledo',
                 'home_abbr': 'toledo',
                 'home_score': 98,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Toledo',
                 'winning_abbr': 'toledo',
                 'losing_name': "St. Joseph's",
                 'losing_abbr': 'saint-josephs'},
                {'boxscore': '2017-11-11-20-nebraska',
                 'away_name': 'Eastern Illinois',
                 'away_abbr': 'eastern-illinois',
                 'away_score': 68,
                 'away_rank': None,
                 'home_name': 'Nebraska',
                 'home_abbr': 'nebraska',
                 'home_score': 72,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Nebraska',
                 'winning_abbr': 'nebraska',
                 'losing_name': 'Eastern Illinois',
                 'losing_abbr': 'eastern-illinois'},
                {'boxscore': '2017-11-11-20-saint-marys-ca',
                 'away_name': 'Saint Francis (PA)',
                 'away_abbr': 'saint-francis-pa',
                 'away_score': 68,
                 'away_rank': None,
                 'home_name': "Saint Mary's (CA)",
                 'home_abbr': 'saint-marys-ca',
                 'home_score': 85,
                 'home_rank': 22,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': "Saint Mary's (CA)",
                 'winning_abbr': 'saint-marys-ca',
                 'losing_name': 'Saint Francis (PA)',
                 'losing_abbr': 'saint-francis-pa'},
                {'boxscore': '2017-11-11-20-texas-arlington',
                 'away_name': 'Loyola Marymount',
                 'away_abbr': 'loyola-marymount',
                 'away_score': 80,
                 'away_rank': None,
                 'home_name': 'Texas-Arlington',
                 'home_abbr': 'texas-arlington',
                 'home_score': 85,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Texas-Arlington',
                 'winning_abbr': 'texas-arlington',
                 'losing_name': 'Loyola Marymount',
                 'losing_abbr': 'loyola-marymount'},
                {'boxscore': '2017-11-11-20-western-illinois',
                 'away_name': "Saint Mary's (MN)",
                 'away_abbr': "Saint Mary's (MN)",
                 'away_score': 68,
                 'away_rank': None,
                 'home_name': 'Western Illinois',
                 'home_abbr': 'western-illinois',
                 'home_score': 109,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Western Illinois',
                 'winning_abbr': 'western-illinois',
                 'losing_name': "Saint Mary's (MN)",
                 'losing_abbr': "Saint Mary's (MN)"},
                {'boxscore': '2017-11-11-21-brigham-young',
                 'away_name': 'Mississippi Valley State',
                 'away_abbr': 'mississippi-valley-state',
                 'away_score': 61,
                 'away_rank': None,
                 'home_name': 'BYU',
                 'home_abbr': 'brigham-young',
                 'home_score': 91,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'BYU',
                 'winning_abbr': 'brigham-young',
                 'losing_name': 'Mississippi Valley State',
                 'losing_abbr': 'mississippi-valley-state'},
                {'boxscore': '2017-11-11-21-kent-state',
                 'away_name': 'Youngstown State',
                 'away_abbr': 'youngstown-state',
                 'away_score': 78,
                 'away_rank': None,
                 'home_name': 'Kent State',
                 'home_abbr': 'kent-state',
                 'home_score': 111,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Kent State',
                 'winning_abbr': 'kent-state',
                 'losing_name': 'Youngstown State',
                 'losing_abbr': 'youngstown-state'},
                {'boxscore': '2017-11-11-21-new-mexico',
                 'away_name': 'Northern New Mexico',
                 'away_abbr': 'Northern New Mexico',
                 'away_score': 76,
                 'away_rank': None,
                 'home_name': 'New Mexico',
                 'home_abbr': 'new-mexico',
                 'home_score': 147,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'New Mexico',
                 'winning_abbr': 'new-mexico',
                 'losing_name': 'Northern New Mexico',
                 'losing_abbr': 'Northern New Mexico'},
                {'boxscore': '2017-11-11-22-nevada-las-vegas',
                 'away_name': 'Florida A&M',
                 'away_abbr': 'florida-am',
                 'away_score': 66,
                 'away_rank': None,
                 'home_name': 'UNLV',
                 'home_abbr': 'nevada-las-vegas',
                 'home_score': 108,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'UNLV',
                 'winning_abbr': 'nevada-las-vegas',
                 'losing_name': 'Florida A&M',
                 'losing_abbr': 'florida-am'},
                {'boxscore': '2017-11-11-22-portland',
                 'away_name': 'Portland State',
                 'away_abbr': 'portland-state',
                 'away_score': 80,
                 'away_rank': None,
                 'home_name': 'Portland',
                 'home_abbr': 'portland',
                 'home_score': 75,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Portland State',
                 'winning_abbr': 'portland-state',
                 'losing_name': 'Portland',
                 'losing_abbr': 'portland'}
            ]
        }

    @mock.patch('requests.get', side_effect=mock_pyquery)
    def test_boxscores_search(self, *args, **kwargs):
        result = Boxscores(datetime(2017, 11, 11)).games

        assert result == self.expected

    @mock.patch('requests.get', side_effect=mock_pyquery)
    def test_boxscores_search_invalid_end(self, *args, **kwargs):
        result = Boxscores(datetime(2017, 11, 11),
                           datetime(2017, 11, 10)).games

        assert result == self.expected

    @mock.patch('requests.get', side_effect=mock_pyquery)
    def test_boxscores_search_multiple_days(self, *args, **kwargs):
        expected = {
            '11-11-2017': [
                {'boxscore': '2017-11-11-13-buffalo',
                 'away_name': 'Canisius',
                 'away_abbr': 'canisius',
                 'away_score': 75,
                 'away_rank': None,
                 'home_name': 'Buffalo',
                 'home_abbr': 'buffalo',
                 'home_score': 80,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Buffalo',
                 'winning_abbr': 'buffalo',
                 'losing_name': 'Canisius',
                 'losing_abbr': 'canisius'},
                {'boxscore': '2017-11-11-13-fairfield',
                 'away_name': 'Penn',
                 'away_abbr': 'pennsylvania',
                 'away_score': 72,
                 'away_rank': None,
                 'home_name': 'Fairfield',
                 'home_abbr': 'fairfield',
                 'home_score': 80,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Fairfield',
                 'winning_abbr': 'fairfield',
                 'losing_name': 'Penn',
                 'losing_abbr': 'pennsylvania'},
                {'boxscore': '2017-11-11-13-florida-gulf-coast',
                 'away_name': 'Illinois State',
                 'away_abbr': 'illinois-state',
                 'away_score': 87,
                 'away_rank': None,
                 'home_name': 'Florida Gulf Coast',
                 'home_abbr': 'florida-gulf-coast',
                 'home_score': 98,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Florida Gulf Coast',
                 'winning_abbr': 'florida-gulf-coast',
                 'losing_name': 'Illinois State',
                 'losing_abbr': 'illinois-state'},
                {'boxscore': '2017-11-11-14-bradley',
                 'away_name': 'IUPUI',
                 'away_abbr': 'iupui',
                 'away_score': 53,
                 'away_rank': None,
                 'home_name': 'Bradley',
                 'home_abbr': 'bradley',
                 'home_score': 68,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Bradley',
                 'winning_abbr': 'bradley',
                 'losing_name': 'IUPUI',
                 'losing_abbr': 'iupui'},
                {'boxscore': '2017-11-11-14-ohio',
                 'away_name': 'Alabama A&M',
                 'away_abbr': 'alabama-am',
                 'away_score': 53,
                 'away_rank': None,
                 'home_name': 'Ohio',
                 'home_abbr': 'ohio',
                 'home_score': 61,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Ohio',
                 'winning_abbr': 'ohio',
                 'losing_name': 'Alabama A&M',
                 'losing_abbr': 'alabama-am'},
                {'boxscore': '2017-11-11-14-quinnipiac',
                 'away_name': 'Dartmouth',
                 'away_abbr': 'dartmouth',
                 'away_score': 77,
                 'away_rank': None,
                 'home_name': 'Quinnipiac',
                 'home_abbr': 'quinnipiac',
                 'home_score': 78,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Quinnipiac',
                 'winning_abbr': 'quinnipiac',
                 'losing_name': 'Dartmouth',
                 'losing_abbr': 'dartmouth'},
                {'boxscore': '2017-11-11-14-western-michigan',
                 'away_name': 'Siena Heights',
                 'away_abbr': 'Siena Heights',
                 'away_score': 72,
                 'away_rank': None,
                 'home_name': 'Western Michigan',
                 'home_abbr': 'western-michigan',
                 'home_score': 103,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Western Michigan',
                 'winning_abbr': 'western-michigan',
                 'losing_name': 'Siena Heights',
                 'losing_abbr': 'Siena Heights'},
                {'boxscore': '2017-11-11-15-appalachian-state',
                 'away_name': 'Toccoa Falls',
                 'away_abbr': 'Toccoa Falls',
                 'away_score': 34,
                 'away_rank': None,
                 'home_name': 'Appalachian State',
                 'home_abbr': 'appalachian-state',
                 'home_score': 135,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Appalachian State',
                 'winning_abbr': 'appalachian-state',
                 'losing_name': 'Toccoa Falls',
                 'losing_abbr': 'Toccoa Falls'},
                {'boxscore': '2017-11-11-15-brown',
                 'away_name': 'Johnson & Wales (RI)',
                 'away_abbr': 'Johnson & Wales (RI)',
                 'away_score': 78,
                 'away_rank': None,
                 'home_name': 'Brown',
                 'home_abbr': 'brown',
                 'home_score': 106,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Brown',
                 'winning_abbr': 'brown',
                 'losing_name': 'Johnson & Wales (RI)',
                 'losing_abbr': 'Johnson & Wales (RI)'},
                {'boxscore': '2017-11-11-15-drake',
                 'away_name': 'Coe',
                 'away_abbr': 'Coe',
                 'away_score': 54,
                 'away_rank': None,
                 'home_name': 'Drake',
                 'home_abbr': 'drake',
                 'home_score': 101,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Drake',
                 'winning_abbr': 'drake',
                 'losing_name': 'Coe',
                 'losing_abbr': 'Coe'},
                {'boxscore': '2017-11-11-15-la-salle',
                 'away_name': "St. Peter's",
                 'away_abbr': 'saint-peters',
                 'away_score': 40,
                 'away_rank': None,
                 'home_name': 'La Salle',
                 'home_abbr': 'la-salle',
                 'home_score': 61,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'La Salle',
                 'winning_abbr': 'la-salle',
                 'losing_name': "St. Peter's",
                 'losing_abbr': 'saint-peters'},
                {'boxscore': '2017-11-11-15-longwood',
                 'away_name': 'Stephen F. Austin',
                 'away_abbr': 'stephen-f-austin',
                 'away_score': 74,
                 'away_rank': None,
                 'home_name': 'Longwood',
                 'home_abbr': 'longwood',
                 'home_score': 61,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Stephen F. Austin',
                 'winning_abbr': 'stephen-f-austin',
                 'losing_name': 'Longwood',
                 'losing_abbr': 'longwood'},
                {'boxscore': '2017-11-11-16-arkansas-little-rock',
                 'away_name': 'Ouachita',
                 'away_abbr': 'Ouachita',
                 'away_score': 81,
                 'away_rank': None,
                 'home_name': 'Little Rock',
                 'home_abbr': 'arkansas-little-rock',
                 'home_score': 79,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Ouachita',
                 'winning_abbr': 'Ouachita',
                 'losing_name': 'Little Rock',
                 'losing_abbr': 'arkansas-little-rock'},
                {'boxscore': '2017-11-11-16-california-santa-barbara',
                 'away_name': 'North Dakota State',
                 'away_abbr': 'north-dakota-state',
                 'away_score': 66,
                 'away_rank': None,
                 'home_name': 'UCSB',
                 'home_abbr': 'california-santa-barbara',
                 'home_score': 85,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'UCSB',
                 'winning_abbr': 'california-santa-barbara',
                 'losing_name': 'North Dakota State',
                 'losing_abbr': 'north-dakota-state'},
                {'boxscore': '2017-11-11-16-depaul',
                 'away_name': 'Notre Dame',
                 'away_abbr': 'notre-dame',
                 'away_score': 72,
                 'away_rank': 14,
                 'home_name': 'DePaul',
                 'home_abbr': 'depaul',
                 'home_score': 58,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Notre Dame',
                 'winning_abbr': 'notre-dame',
                 'losing_name': 'DePaul',
                 'losing_abbr': 'depaul'},
                {'boxscore': '2017-11-11-17-duquesne',
                 'away_name': 'St. Francis (NY)',
                 'away_abbr': 'st-francis-ny',
                 'away_score': 70,
                 'away_rank': None,
                 'home_name': 'Duquesne',
                 'home_abbr': 'duquesne',
                 'home_score': 80,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Duquesne',
                 'winning_abbr': 'duquesne',
                 'losing_name': 'St. Francis (NY)',
                 'losing_abbr': 'st-francis-ny'},
                {'boxscore': '2017-11-11-18-njit',
                 'away_name': 'Wagner',
                 'away_abbr': 'wagner',
                 'away_score': 60,
                 'away_rank': None,
                 'home_name': 'NJIT',
                 'home_abbr': 'njit',
                 'home_score': 49,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Wagner',
                 'winning_abbr': 'wagner',
                 'losing_name': 'NJIT',
                 'losing_abbr': 'njit'},
                {'boxscore': '2017-11-11-19-akron',
                 'away_name': 'Cleveland State',
                 'away_abbr': 'cleveland-state',
                 'away_score': 57,
                 'away_rank': None,
                 'home_name': 'Akron',
                 'home_abbr': 'akron',
                 'home_score': 67,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Akron',
                 'winning_abbr': 'akron',
                 'losing_name': 'Cleveland State',
                 'losing_abbr': 'cleveland-state'},
                {'boxscore': '2017-11-11-19-duke',
                 'away_name': 'Utah Valley',
                 'away_abbr': 'utah-valley',
                 'away_score': 69,
                 'away_rank': None,
                 'home_name': 'Duke',
                 'home_abbr': 'duke',
                 'home_score': 99,
                 'home_rank': 1,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Duke',
                 'winning_abbr': 'duke',
                 'losing_name': 'Utah Valley',
                 'losing_abbr': 'utah-valley'},
                {'boxscore': '2017-11-11-19-elon',
                 'away_name': 'Peace',
                 'away_abbr': 'Peace',
                 'away_score': 53,
                 'away_rank': None,
                 'home_name': 'Elon',
                 'home_abbr': 'elon',
                 'home_score': 96,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Elon',
                 'winning_abbr': 'elon',
                 'losing_name': 'Peace',
                 'losing_abbr': 'Peace'},
                {'boxscore': '2017-11-11-19-marist',
                 'away_name': 'Lehigh',
                 'away_abbr': 'lehigh',
                 'away_score': 84,
                 'away_rank': None,
                 'home_name': 'Marist',
                 'home_abbr': 'marist',
                 'home_score': 76,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Lehigh',
                 'winning_abbr': 'lehigh',
                 'losing_name': 'Marist',
                 'losing_abbr': 'marist'},
                {'boxscore': '2017-11-11-19-michigan',
                 'away_name': 'North Florida',
                 'away_abbr': 'north-florida',
                 'away_score': 66,
                 'away_rank': None,
                 'home_name': 'Michigan',
                 'home_abbr': 'michigan',
                 'home_score': 86,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Michigan',
                 'winning_abbr': 'michigan',
                 'losing_name': 'North Florida',
                 'losing_abbr': 'north-florida'},
                {'boxscore': '2017-11-11-19-north-carolina-wilmington',
                 'away_name': 'North Carolina Wesleyan',
                 'away_abbr': 'North Carolina Wesleyan',
                 'away_score': 81,
                 'away_rank': None,
                 'home_name': 'North Carolina-Wilmington',
                 'home_abbr': 'north-carolina-wilmington',
                 'home_score': 105,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'North Carolina-Wilmington',
                 'winning_abbr': 'north-carolina-wilmington',
                 'losing_name': 'North Carolina Wesleyan',
                 'losing_abbr': 'North Carolina Wesleyan'},
                {'boxscore': '2017-11-11-19-presbyterian',
                 'away_name': 'Johnson University',
                 'away_abbr': 'Johnson University',
                 'away_score': 87,
                 'away_rank': None,
                 'home_name': 'Presbyterian',
                 'home_abbr': 'presbyterian',
                 'home_score': 107,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Presbyterian',
                 'winning_abbr': 'presbyterian',
                 'losing_name': 'Johnson University',
                 'losing_abbr': 'Johnson University'},
                {'boxscore': '2017-11-11-19-sam-houston-state',
                 'away_name': 'Texas-Tyler',
                 'away_abbr': 'Texas-Tyler',
                 'away_score': 51,
                 'away_rank': None,
                 'home_name': 'Sam Houston State',
                 'home_abbr': 'sam-houston-state',
                 'home_score': 91,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Sam Houston State',
                 'winning_abbr': 'sam-houston-state',
                 'losing_name': 'Texas-Tyler',
                 'losing_abbr': 'Texas-Tyler'},
                {'boxscore': '2017-11-11-19-south-dakota',
                 'away_name': 'Mayville State',
                 'away_abbr': 'Mayville State',
                 'away_score': 59,
                 'away_rank': None,
                 'home_name': 'South Dakota',
                 'home_abbr': 'south-dakota',
                 'home_score': 87,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'South Dakota',
                 'winning_abbr': 'south-dakota',
                 'losing_name': 'Mayville State',
                 'losing_abbr': 'Mayville State'},
                {'boxscore': '2017-11-11-19-toledo',
                 'away_name': "St. Joseph's",
                 'away_abbr': 'saint-josephs',
                 'away_score': 87,
                 'away_rank': None,
                 'home_name': 'Toledo',
                 'home_abbr': 'toledo',
                 'home_score': 98,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Toledo',
                 'winning_abbr': 'toledo',
                 'losing_name': "St. Joseph's",
                 'losing_abbr': 'saint-josephs'},
                {'boxscore': '2017-11-11-20-nebraska',
                 'away_name': 'Eastern Illinois',
                 'away_abbr': 'eastern-illinois',
                 'away_score': 68,
                 'away_rank': None,
                 'home_name': 'Nebraska',
                 'home_abbr': 'nebraska',
                 'home_score': 72,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Nebraska',
                 'winning_abbr': 'nebraska',
                 'losing_name': 'Eastern Illinois',
                 'losing_abbr': 'eastern-illinois'},
                {'boxscore': '2017-11-11-20-saint-marys-ca',
                 'away_name': 'Saint Francis (PA)',
                 'away_abbr': 'saint-francis-pa',
                 'away_score': 68,
                 'away_rank': None,
                 'home_name': "Saint Mary's (CA)",
                 'home_abbr': 'saint-marys-ca',
                 'home_score': 85,
                 'home_rank': 22,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': "Saint Mary's (CA)",
                 'winning_abbr': 'saint-marys-ca',
                 'losing_name': 'Saint Francis (PA)',
                 'losing_abbr': 'saint-francis-pa'},
                {'boxscore': '2017-11-11-20-texas-arlington',
                 'away_name': 'Loyola Marymount',
                 'away_abbr': 'loyola-marymount',
                 'away_score': 80,
                 'away_rank': None,
                 'home_name': 'Texas-Arlington',
                 'home_abbr': 'texas-arlington',
                 'home_score': 85,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Texas-Arlington',
                 'winning_abbr': 'texas-arlington',
                 'losing_name': 'Loyola Marymount',
                 'losing_abbr': 'loyola-marymount'},
                {'boxscore': '2017-11-11-20-western-illinois',
                 'away_name': "Saint Mary's (MN)",
                 'away_abbr': "Saint Mary's (MN)",
                 'away_score': 68,
                 'away_rank': None,
                 'home_name': 'Western Illinois',
                 'home_abbr': 'western-illinois',
                 'home_score': 109,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Western Illinois',
                 'winning_abbr': 'western-illinois',
                 'losing_name': "Saint Mary's (MN)",
                 'losing_abbr': "Saint Mary's (MN)"},
                {'boxscore': '2017-11-11-21-brigham-young',
                 'away_name': 'Mississippi Valley State',
                 'away_abbr': 'mississippi-valley-state',
                 'away_score': 61,
                 'away_rank': None,
                 'home_name': 'BYU',
                 'home_abbr': 'brigham-young',
                 'home_score': 91,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'BYU',
                 'winning_abbr': 'brigham-young',
                 'losing_name': 'Mississippi Valley State',
                 'losing_abbr': 'mississippi-valley-state'},
                {'boxscore': '2017-11-11-21-kent-state',
                 'away_name': 'Youngstown State',
                 'away_abbr': 'youngstown-state',
                 'away_score': 78,
                 'away_rank': None,
                 'home_name': 'Kent State',
                 'home_abbr': 'kent-state',
                 'home_score': 111,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Kent State',
                 'winning_abbr': 'kent-state',
                 'losing_name': 'Youngstown State',
                 'losing_abbr': 'youngstown-state'},
                {'boxscore': '2017-11-11-21-new-mexico',
                 'away_name': 'Northern New Mexico',
                 'away_abbr': 'Northern New Mexico',
                 'away_score': 76,
                 'away_rank': None,
                 'home_name': 'New Mexico',
                 'home_abbr': 'new-mexico',
                 'home_score': 147,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'New Mexico',
                 'winning_abbr': 'new-mexico',
                 'losing_name': 'Northern New Mexico',
                 'losing_abbr': 'Northern New Mexico'},
                {'boxscore': '2017-11-11-22-nevada-las-vegas',
                 'away_name': 'Florida A&M',
                 'away_abbr': 'florida-am',
                 'away_score': 66,
                 'away_rank': None,
                 'home_name': 'UNLV',
                 'home_abbr': 'nevada-las-vegas',
                 'home_score': 108,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'UNLV',
                 'winning_abbr': 'nevada-las-vegas',
                 'losing_name': 'Florida A&M',
                 'losing_abbr': 'florida-am'},
                {'boxscore': '2017-11-11-22-portland',
                 'away_name': 'Portland State',
                 'away_abbr': 'portland-state',
                 'away_score': 80,
                 'away_rank': None,
                 'home_name': 'Portland',
                 'home_abbr': 'portland',
                 'home_score': 75,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Portland State',
                 'winning_abbr': 'portland-state',
                 'losing_name': 'Portland',
                 'losing_abbr': 'portland'}
            ],
            '11-12-2017': [
                {'boxscore': '2017-11-12-12-georgetown',
                 'away_name': 'Jacksonville',
                 'away_abbr': 'jacksonville',
                 'away_score': 57,
                 'away_rank': None,
                 'home_name': 'Georgetown',
                 'home_abbr': 'georgetown',
                 'home_score': 73,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Georgetown',
                 'winning_abbr': 'georgetown',
                 'losing_name': 'Jacksonville',
                 'losing_abbr': 'jacksonville'},
                {'boxscore': '2017-11-12-13-boston-college',
                 'away_name': 'South Carolina State',
                 'away_abbr': 'south-carolina-state',
                 'away_score': 52,
                 'away_rank': None,
                 'home_name': 'Boston College',
                 'home_abbr': 'boston-college',
                 'home_score': 91,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Boston College',
                 'winning_abbr': 'boston-college',
                 'losing_name': 'South Carolina State',
                 'losing_abbr': 'south-carolina-state'},
                {'boxscore': '2017-11-12-13-massachusetts-lowell',
                 'away_name': 'Massachusetts-Boston',
                 'away_abbr': 'Massachusetts-Boston',
                 'away_score': 63,
                 'away_rank': None,
                 'home_name': 'UMass-Lowell',
                 'home_abbr': 'massachusetts-lowell',
                 'home_score': 102,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'UMass-Lowell',
                 'winning_abbr': 'massachusetts-lowell',
                 'losing_name': 'Massachusetts-Boston',
                 'losing_abbr': 'Massachusetts-Boston'},
                {'boxscore': '2017-11-12-13-northern-kentucky',
                 'away_name': 'Wilmington (OH)',
                 'away_abbr': 'Wilmington (OH)',
                 'away_score': 65,
                 'away_rank': None,
                 'home_name': 'Northern Kentucky',
                 'home_abbr': 'northern-kentucky',
                 'home_score': 93,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Northern Kentucky',
                 'winning_abbr': 'northern-kentucky',
                 'losing_name': 'Wilmington (OH)',
                 'losing_abbr': 'Wilmington (OH)'},
                {'boxscore': '2017-11-12-13-seton-hall',
                 'away_name': 'Monmouth',
                 'away_abbr': 'monmouth',
                 'away_score': 65,
                 'away_rank': None,
                 'home_name': 'Seton Hall',
                 'home_abbr': 'seton-hall',
                 'home_score': 75,
                 'home_rank': 23,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Seton Hall',
                 'winning_abbr': 'seton-hall',
                 'losing_name': 'Monmouth',
                 'losing_abbr': 'monmouth'},
                {'boxscore': '2017-11-12-14-clemson',
                 'away_name': 'North Carolina A&T',
                 'away_abbr': 'north-carolina-at',
                 'away_score': 63,
                 'away_rank': None,
                 'home_name': 'Clemson',
                 'home_abbr': 'clemson',
                 'home_score': 87,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Clemson',
                 'winning_abbr': 'clemson',
                 'losing_name': 'North Carolina A&T',
                 'losing_abbr': 'north-carolina-at'},
                {'boxscore': '2017-11-12-14-eastern-michigan',
                 'away_name': 'Michigan-Dearborn',
                 'away_abbr': 'Michigan-Dearborn',
                 'away_score': 73,
                 'away_rank': None,
                 'home_name': 'Eastern Michigan',
                 'home_abbr': 'eastern-michigan',
                 'home_score': 97,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Eastern Michigan',
                 'winning_abbr': 'eastern-michigan',
                 'losing_name': 'Michigan-Dearborn',
                 'losing_abbr': 'Michigan-Dearborn'},
                {'boxscore': '2017-11-12-14-harvard',
                 'away_name': 'UMass',
                 'away_abbr': 'massachusetts',
                 'away_score': 67,
                 'away_rank': None,
                 'home_name': 'Harvard',
                 'home_abbr': 'harvard',
                 'home_score': 70,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Harvard',
                 'winning_abbr': 'harvard',
                 'losing_name': 'UMass',
                 'losing_abbr': 'massachusetts'},
                {'boxscore': '2017-11-12-14-louisville',
                 'away_name': 'George Mason',
                 'away_abbr': 'george-mason',
                 'away_score': 61,
                 'away_rank': None,
                 'home_name': 'Louisville',
                 'home_abbr': 'louisville',
                 'home_score': 72,
                 'home_rank': 16,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Louisville',
                 'winning_abbr': 'louisville',
                 'losing_name': 'George Mason',
                 'losing_abbr': 'george-mason'},
                {'boxscore': '2017-11-12-14-mercer',
                 'away_name': 'Toccoa Falls',
                 'away_abbr': 'Toccoa Falls',
                 'away_score': 43,
                 'away_rank': None,
                 'home_name': 'Mercer',
                 'home_abbr': 'mercer',
                 'home_score': 110,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Mercer',
                 'winning_abbr': 'mercer',
                 'losing_name': 'Toccoa Falls',
                 'losing_abbr': 'Toccoa Falls'},
                {'boxscore': '2017-11-12-14-ohio-state',
                 'away_name': 'Radford',
                 'away_abbr': 'radford',
                 'away_score': 72,
                 'away_rank': None,
                 'home_name': 'Ohio State',
                 'home_abbr': 'ohio-state',
                 'home_score': 82,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Ohio State',
                 'winning_abbr': 'ohio-state',
                 'losing_name': 'Radford',
                 'losing_abbr': 'radford'},
                {'boxscore': '2017-11-12-14-rutgers',
                 'away_name': 'Central Connecticut',
                 'away_abbr': 'central-connecticut-state',
                 'away_score': 67,
                 'away_rank': None,
                 'home_name': 'Rutgers',
                 'home_abbr': 'rutgers',
                 'home_score': 71,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Rutgers',
                 'winning_abbr': 'rutgers',
                 'losing_name': 'Central Connecticut',
                 'losing_abbr': 'central-connecticut-state'},
                {'boxscore': '2017-11-12-14-wofford',
                 'away_name': 'Washington & Lee',
                 'away_abbr': 'Washington & Lee',
                 'away_score': 66,
                 'away_rank': None,
                 'home_name': 'Wofford',
                 'home_abbr': 'wofford',
                 'home_score': 103,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Wofford',
                 'winning_abbr': 'wofford',
                 'losing_name': 'Washington & Lee',
                 'losing_abbr': 'Washington & Lee'},
                {'boxscore': '2017-11-12-15-appalachian-state',
                 'away_name': 'Bridgewater (VA)',
                 'away_abbr': 'Bridgewater (VA)',
                 'away_score': 57,
                 'away_rank': None,
                 'home_name': 'Appalachian State',
                 'home_abbr': 'appalachian-state',
                 'home_score': 95,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Appalachian State',
                 'winning_abbr': 'appalachian-state',
                 'losing_name': 'Bridgewater (VA)',
                 'losing_abbr': 'Bridgewater (VA)'},
                {'boxscore': '2017-11-12-15-kentucky',
                 'away_name': 'Vermont',
                 'away_abbr': 'vermont',
                 'away_score': 69,
                 'away_rank': None,
                 'home_name': 'Kentucky',
                 'home_abbr': 'kentucky',
                 'home_score': 73,
                 'home_rank': 5,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Kentucky',
                 'winning_abbr': 'kentucky',
                 'losing_name': 'Vermont',
                 'losing_abbr': 'vermont'},
                {'boxscore': '2017-11-12-15-loyola-il',
                 'away_name': 'Eureka',
                 'away_abbr': 'Eureka',
                 'away_score': 69,
                 'away_rank': None,
                 'home_name': 'Loyola (IL)',
                 'home_abbr': 'loyola-il',
                 'home_score': 96,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Loyola (IL)',
                 'winning_abbr': 'loyola-il',
                 'losing_name': 'Eureka',
                 'losing_abbr': 'Eureka'},
                {'boxscore': '2017-11-12-15-oklahoma',
                 'away_name': 'Omaha',
                 'away_abbr': 'nebraska-omaha',
                 'away_score': 89,
                 'away_rank': None,
                 'home_name': 'Oklahoma',
                 'home_abbr': 'oklahoma',
                 'home_score': 108,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Oklahoma',
                 'winning_abbr': 'oklahoma',
                 'losing_name': 'Omaha',
                 'losing_abbr': 'nebraska-omaha'},
                {'boxscore': '2017-11-12-15-south-carolina-upstate',
                 'away_name': 'Allen',
                 'away_abbr': 'Allen',
                 'away_score': 66,
                 'away_rank': None,
                 'home_name': 'USC Upstate',
                 'home_abbr': 'south-carolina-upstate',
                 'home_score': 85,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'USC Upstate',
                 'winning_abbr': 'south-carolina-upstate',
                 'losing_name': 'Allen',
                 'losing_abbr': 'Allen'},
                {'boxscore': '2017-11-12-15-south-dakota-state',
                 'away_name': 'Mary',
                 'away_abbr': 'Mary',
                 'away_score': 54,
                 'away_rank': None,
                 'home_name': 'South Dakota State',
                 'home_abbr': 'south-dakota-state',
                 'home_score': 78,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'South Dakota State',
                 'winning_abbr': 'south-dakota-state',
                 'losing_name': 'Mary',
                 'losing_abbr': 'Mary'},
                {'boxscore': '2017-11-12-16-air-force',
                 'away_name': 'Texas State',
                 'away_abbr': 'texas-state',
                 'away_score': 57,
                 'away_rank': None,
                 'home_name': 'Air Force',
                 'home_abbr': 'air-force',
                 'home_score': 65,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Air Force',
                 'winning_abbr': 'air-force',
                 'losing_name': 'Texas State',
                 'losing_abbr': 'texas-state'},
                {'boxscore': '2017-11-12-16-boise-state',
                 'away_name': 'Southern Utah',
                 'away_abbr': 'southern-utah',
                 'away_score': 69,
                 'away_rank': None,
                 'home_name': 'Boise State',
                 'home_abbr': 'boise-state',
                 'home_score': 90,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Boise State',
                 'winning_abbr': 'boise-state',
                 'losing_name': 'Southern Utah',
                 'losing_abbr': 'southern-utah'},
                {'boxscore': '2017-11-12-16-california',
                 'away_name': 'Cal Poly',
                 'away_abbr': 'cal-poly',
                 'away_score': 82,
                 'away_rank': None,
                 'home_name': 'California',
                 'home_abbr': 'california',
                 'home_score': 85,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'California',
                 'winning_abbr': 'california',
                 'losing_name': 'Cal Poly',
                 'losing_abbr': 'cal-poly'},
                {'boxscore': '2017-11-12-16-hartford',
                 'away_name': 'Gordon',
                 'away_abbr': 'Gordon',
                 'away_score': 63,
                 'away_rank': None,
                 'home_name': 'Hartford',
                 'home_abbr': 'hartford',
                 'home_score': 80,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Hartford',
                 'winning_abbr': 'hartford',
                 'losing_name': 'Gordon',
                 'losing_abbr': 'Gordon'},
                {'boxscore': '2017-11-12-16-iowa',
                 'away_name': 'Alabama State',
                 'away_abbr': 'alabama-state',
                 'away_score': 58,
                 'away_rank': None,
                 'home_name': 'Iowa',
                 'home_abbr': 'iowa',
                 'home_score': 92,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Iowa',
                 'winning_abbr': 'iowa',
                 'losing_name': 'Alabama State',
                 'losing_abbr': 'alabama-state'},
                {'boxscore': '2017-11-12-16-ipfw',
                 'away_name': 'Defiance',
                 'away_abbr': 'Defiance',
                 'away_score': 59,
                 'away_rank': None,
                 'home_name': 'Purdue-Fort Wayne',
                 'home_abbr': 'ipfw',
                 'home_score': 114,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Purdue-Fort Wayne',
                 'winning_abbr': 'ipfw',
                 'losing_name': 'Defiance',
                 'losing_abbr': 'Defiance'},
                {'boxscore': '2017-11-12-16-miami-fl',
                 'away_name': 'Navy',
                 'away_abbr': 'navy',
                 'away_score': 55,
                 'away_rank': None,
                 'home_name': 'Miami (FL)',
                 'home_abbr': 'miami-fl',
                 'home_score': 89,
                 'home_rank': 13,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Miami (FL)',
                 'winning_abbr': 'miami-fl',
                 'losing_name': 'Navy',
                 'losing_abbr': 'navy'},
                {'boxscore': '2017-11-12-16-north-carolina-state',
                 'away_name': 'Charleston Southern',
                 'away_abbr': 'charleston-southern',
                 'away_score': 56,
                 'away_rank': None,
                 'home_name': 'NC State',
                 'home_abbr': 'north-carolina-state',
                 'home_score': 78,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'NC State',
                 'winning_abbr': 'north-carolina-state',
                 'losing_name': 'Charleston Southern',
                 'losing_abbr': 'charleston-southern'},
                {'boxscore': '2017-11-12-16-northeastern',
                 'away_name': 'Wentworth',
                 'away_abbr': 'Wentworth',
                 'away_score': 49,
                 'away_rank': None,
                 'home_name': 'Northeastern',
                 'home_abbr': 'northeastern',
                 'home_score': 83,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Northeastern',
                 'winning_abbr': 'northeastern',
                 'losing_name': 'Wentworth',
                 'losing_abbr': 'Wentworth'},
                {'boxscore': '2017-11-12-16-purdue',
                 'away_name': 'Chicago State',
                 'away_abbr': 'chicago-state',
                 'away_score': 42,
                 'away_rank': None,
                 'home_name': 'Purdue',
                 'home_abbr': 'purdue',
                 'home_score': 111,
                 'home_rank': 20,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Purdue',
                 'winning_abbr': 'purdue',
                 'losing_name': 'Chicago State',
                 'losing_abbr': 'chicago-state'},
                {'boxscore': '2017-11-12-16-southern-methodist',
                 'away_name': 'Louisiana-Monroe',
                 'away_abbr': 'louisiana-monroe',
                 'away_score': 65,
                 'away_rank': None,
                 'home_name': 'SMU',
                 'home_abbr': 'southern-methodist',
                 'home_score': 83,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'SMU',
                 'winning_abbr': 'southern-methodist',
                 'losing_name': 'Louisiana-Monroe',
                 'losing_abbr': 'louisiana-monroe'},
                {'boxscore': '2017-11-12-16-texas-san-antonio',
                 'away_name': 'East Central',
                 'away_abbr': 'East Central',
                 'away_score': 59,
                 'away_rank': None,
                 'home_name': 'UTSA',
                 'home_abbr': 'texas-san-antonio',
                 'home_score': 92,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'UTSA',
                 'winning_abbr': 'texas-san-antonio',
                 'losing_name': 'East Central',
                 'losing_abbr': 'East Central'},
                {'boxscore': '2017-11-12-16-washington-state',
                 'away_name': 'Texas Southern',
                 'away_abbr': 'texas-southern',
                 'away_score': 84,
                 'away_rank': None,
                 'home_name': 'Washington State',
                 'home_abbr': 'washington-state',
                 'home_score': 86,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Washington State',
                 'winning_abbr': 'washington-state',
                 'losing_name': 'Texas Southern',
                 'losing_abbr': 'texas-southern'},
                {'boxscore': '2017-11-12-17-central-arkansas',
                 'away_name': 'University of the Ozarks',
                 'away_abbr': 'University of the Ozarks',
                 'away_score': 51,
                 'away_rank': None,
                 'home_name': 'Central Arkansas',
                 'home_abbr': 'central-arkansas',
                 'home_score': 99,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Central Arkansas',
                 'winning_abbr': 'central-arkansas',
                 'losing_name': 'University of the Ozarks',
                 'losing_abbr': 'University of the Ozarks'},
                {'boxscore': '2017-11-12-17-east-carolina',
                 'away_name': 'Coppin State',
                 'away_abbr': 'coppin-state',
                 'away_score': 50,
                 'away_rank': None,
                 'home_name': 'East Carolina',
                 'home_abbr': 'east-carolina',
                 'home_score': 76,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'East Carolina',
                 'winning_abbr': 'east-carolina',
                 'losing_name': 'Coppin State',
                 'losing_abbr': 'coppin-state'},
                {'boxscore': '2017-11-12-17-hampton',
                 'away_name': 'Bluefield College (VA)',
                 'away_abbr': 'Bluefield College (VA)',
                 'away_score': 68,
                 'away_rank': None,
                 'home_name': 'Hampton',
                 'home_abbr': 'hampton',
                 'home_score': 85,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Hampton',
                 'winning_abbr': 'hampton',
                 'losing_name': 'Bluefield College (VA)',
                 'losing_abbr': 'Bluefield College (VA)'},
                {'boxscore': '2017-11-12-17-lipscomb',
                 'away_name': 'Morehead State',
                 'away_abbr': 'morehead-state',
                 'away_score': 70,
                 'away_rank': None,
                 'home_name': 'Lipscomb',
                 'home_abbr': 'lipscomb',
                 'home_score': 77,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Lipscomb',
                 'winning_abbr': 'lipscomb',
                 'losing_name': 'Morehead State',
                 'losing_abbr': 'morehead-state'},
                {'boxscore': '2017-11-12-17-missouri-kansas-city',
                 'away_name': 'Haskell Indian Nations Institute',
                 'away_abbr': 'Haskell Indian Nations Institute',
                 'away_score': 59,
                 'away_rank': None,
                 'home_name': 'UMKC',
                 'home_abbr': 'missouri-kansas-city',
                 'home_score': 110,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'UMKC',
                 'winning_abbr': 'missouri-kansas-city',
                 'losing_name': 'Haskell Indian Nations Institute',
                 'losing_abbr': 'Haskell Indian Nations Institute'},
                {'boxscore': '2017-11-12-17-northwestern-state',
                 'away_name': 'LeTourneau',
                 'away_abbr': 'LeTourneau',
                 'away_score': 99,
                 'away_rank': None,
                 'home_name': 'Northwestern State',
                 'home_abbr': 'northwestern-state',
                 'home_score': 84,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'LeTourneau',
                 'winning_abbr': 'LeTourneau',
                 'losing_name': 'Northwestern State',
                 'losing_abbr': 'northwestern-state'},
                {'boxscore': '2017-11-12-17-penn-state',
                 'away_name': 'Fairleigh Dickinson',
                 'away_abbr': 'fairleigh-dickinson',
                 'away_score': 57,
                 'away_rank': None,
                 'home_name': 'Penn State',
                 'home_abbr': 'penn-state',
                 'home_score': 81,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Penn State',
                 'winning_abbr': 'penn-state',
                 'losing_name': 'Fairleigh Dickinson',
                 'losing_abbr': 'fairleigh-dickinson'},
                {'boxscore': '2017-11-12-17-san-jose-state',
                 'away_name': 'San Diego',
                 'away_abbr': 'san-diego',
                 'away_score': 81,
                 'away_rank': None,
                 'home_name': 'San Jose State',
                 'home_abbr': 'san-jose-state',
                 'home_score': 64,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'San Diego',
                 'winning_abbr': 'san-diego',
                 'losing_name': 'San Jose State',
                 'losing_abbr': 'san-jose-state'},
                {'boxscore': '2017-11-12-18-arizona',
                 'away_name': 'UMBC',
                 'away_abbr': 'maryland-baltimore-county',
                 'away_score': 78,
                 'away_rank': None,
                 'home_name': 'Arizona',
                 'home_abbr': 'arizona',
                 'home_score': 103,
                 'home_rank': 3,
                 'non_di': False,
                 'top_25': True,
                 'winning_name': 'Arizona',
                 'winning_abbr': 'arizona',
                 'losing_name': 'UMBC',
                 'losing_abbr': 'maryland-baltimore-county'},
                {'boxscore': '2017-11-12-18-arkansas',
                 'away_name': 'Bucknell',
                 'away_abbr': 'bucknell',
                 'away_score': 73,
                 'away_rank': None,
                 'home_name': 'Arkansas',
                 'home_abbr': 'arkansas',
                 'home_score': 101,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Arkansas',
                 'winning_abbr': 'arkansas',
                 'losing_name': 'Bucknell',
                 'losing_abbr': 'bucknell'},
                {'boxscore': '2017-11-12-18-butler',
                 'away_name': 'Princeton',
                 'away_abbr': 'princeton',
                 'away_score': 75,
                 'away_rank': None,
                 'home_name': 'Butler',
                 'home_abbr': 'butler',
                 'home_score': 85,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Butler',
                 'winning_abbr': 'butler',
                 'losing_name': 'Princeton',
                 'losing_abbr': 'princeton'},
                {'boxscore': '2017-11-12-18-denver',
                 'away_name': 'UC-Irvine',
                 'away_abbr': 'california-irvine',
                 'away_score': 83,
                 'away_rank': None,
                 'home_name': 'Denver',
                 'home_abbr': 'denver',
                 'home_score': 69,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'UC-Irvine',
                 'winning_abbr': 'california-irvine',
                 'losing_name': 'Denver',
                 'losing_abbr': 'denver'},
                {'boxscore': '2017-11-12-18-illinois',
                 'away_name': 'UT-Martin',
                 'away_abbr': 'tennessee-martin',
                 'away_score': 74,
                 'away_rank': None,
                 'home_name': 'Illinois',
                 'home_abbr': 'illinois',
                 'home_score': 77,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Illinois',
                 'winning_abbr': 'illinois',
                 'losing_name': 'UT-Martin',
                 'losing_abbr': 'tennessee-martin'},
                {'boxscore': '2017-11-12-18-maryland',
                 'away_name': 'Maryland-Eastern Shore',
                 'away_abbr': 'maryland-eastern-shore',
                 'away_score': 43,
                 'away_rank': None,
                 'home_name': 'Maryland',
                 'home_abbr': 'maryland',
                 'home_score': 96,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Maryland',
                 'winning_abbr': 'maryland',
                 'losing_name': 'Maryland-Eastern Shore',
                 'losing_abbr': 'maryland-eastern-shore'},
                {'boxscore': '2017-11-12-18-wisconsin',
                 'away_name': 'Yale',
                 'away_abbr': 'yale',
                 'away_score': 61,
                 'away_rank': None,
                 'home_name': 'Wisconsin',
                 'home_abbr': 'wisconsin',
                 'home_score': 89,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Wisconsin',
                 'winning_abbr': 'wisconsin',
                 'losing_name': 'Yale',
                 'losing_abbr': 'yale'},
                {'boxscore': '2017-11-12-19-arkansas-pine-bluff',
                 'away_name': 'Troy',
                 'away_abbr': 'troy',
                 'away_score': 81,
                 'away_rank': None,
                 'home_name': 'Arkansas-Pine Bluff',
                 'home_abbr': 'arkansas-pine-bluff',
                 'home_score': 57,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Troy',
                 'winning_abbr': 'troy',
                 'losing_name': 'Arkansas-Pine Bluff',
                 'losing_abbr': 'arkansas-pine-bluff'},
                {'boxscore': '2017-11-12-19-creighton',
                 'away_name': 'Alcorn State',
                 'away_abbr': 'alcorn-state',
                 'away_score': 72,
                 'away_rank': None,
                 'home_name': 'Creighton',
                 'home_abbr': 'creighton',
                 'home_score': 109,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Creighton',
                 'winning_abbr': 'creighton',
                 'losing_name': 'Alcorn State',
                 'losing_abbr': 'alcorn-state'},
                {'boxscore': '2017-11-12-19-florida-international',
                 'away_name': 'Stetson',
                 'away_abbr': 'stetson',
                 'away_score': 70,
                 'away_rank': None,
                 'home_name': 'Florida International',
                 'home_abbr': 'florida-international',
                 'home_score': 64,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Stetson',
                 'winning_abbr': 'stetson',
                 'losing_name': 'Florida International',
                 'losing_abbr': 'florida-international'},
                {'boxscore': '2017-11-12-19-seattle',
                 'away_name': 'Puget Sound',
                 'away_abbr': 'Puget Sound',
                 'away_score': 70,
                 'away_rank': None,
                 'home_name': 'Seattle',
                 'home_abbr': 'seattle',
                 'home_score': 121,
                 'home_rank': None,
                 'non_di': True,
                 'top_25': False,
                 'winning_name': 'Seattle',
                 'winning_abbr': 'seattle',
                 'losing_name': 'Puget Sound',
                 'losing_abbr': 'Puget Sound'},
                {'boxscore': '2017-11-12-19-virginia-tech',
                 'away_name': 'Citadel',
                 'away_abbr': 'citadel',
                 'away_score': 93,
                 'away_rank': None,
                 'home_name': 'Virginia Tech',
                 'home_abbr': 'virginia-tech',
                 'home_score': 132,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Virginia Tech',
                 'winning_abbr': 'virginia-tech',
                 'losing_name': 'Citadel',
                 'losing_abbr': 'citadel'},
                {'boxscore': '2017-11-12-20-indiana',
                 'away_name': 'Howard',
                 'away_abbr': 'howard',
                 'away_score': 77,
                 'away_rank': None,
                 'home_name': 'Indiana',
                 'home_abbr': 'indiana',
                 'home_score': 86,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Indiana',
                 'winning_abbr': 'indiana',
                 'losing_name': 'Howard',
                 'losing_abbr': 'howard'},
                {'boxscore': '2017-11-12-20-washington',
                 'away_name': 'Eastern Washington',
                 'away_abbr': 'eastern-washington',
                 'away_score': 69,
                 'away_rank': None,
                 'home_name': 'Washington',
                 'home_abbr': 'washington',
                 'home_score': 79,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Washington',
                 'winning_abbr': 'washington',
                 'losing_name': 'Eastern Washington',
                 'losing_abbr': 'eastern-washington'},
                {'boxscore': '2017-11-12-22-hawaii',
                 'away_name': 'North Dakota',
                 'away_abbr': 'north-dakota',
                 'away_score': 78,
                 'away_rank': None,
                 'home_name': 'Hawaii',
                 'home_abbr': 'hawaii',
                 'home_score': 81,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Hawaii',
                 'winning_abbr': 'hawaii',
                 'losing_name': 'North Dakota',
                 'losing_abbr': 'north-dakota'},
                {'boxscore': '2017-11-12-22-stanford',
                 'away_name': 'Pacific',
                 'away_abbr': 'pacific',
                 'away_score': 80,
                 'away_rank': None,
                 'home_name': 'Stanford',
                 'home_abbr': 'stanford',
                 'home_score': 89,
                 'home_rank': None,
                 'non_di': False,
                 'top_25': False,
                 'winning_name': 'Stanford',
                 'winning_abbr': 'stanford',
                 'losing_name': 'Pacific',
                 'losing_abbr': 'pacific'}
                ]
            }



import mock
import os
import pandas as pd
import pytest
from flexmock import flexmock
from sportsreference import utils
from sportsreference.mlb.constants import STANDINGS_URL, TEAM_STATS_URL
from sportsreference.mlb.teams import Teams


MONTH = 4
YEAR = 2017


def read_file(filename):
    filepath = os.path.join(os.path.dirname(__file__), 'mlb_stats', filename)
    return open('%s' % filepath, 'r').read()


def mock_pyquery(url):
    class MockPQ:
        def __init__(self, html_contents):
            self.status_code = 200
            self.html_contents = html_contents
            self.text = html_contents

        def __call__(self, div):
            if div == 'div#all_teams_standard_batting':
                return read_file('%s_batting.html' % YEAR)
            elif div == 'div#all_teams_standard_pitching':
                return read_file('%s_pitching.html' % YEAR)
            else:
                return read_file('%s_overall.html' % YEAR)

    html_contents = read_file('%s-standings.html' % YEAR)
    team_stats = read_file('%s.html' % YEAR)

    if url == STANDINGS_URL % YEAR:
        return MockPQ(html_contents)
    elif url == TEAM_STATS_URL % YEAR:
        return MockPQ(team_stats)


class MockDateTime:
    def __init__(self, year, month):
        self.year = year
        self.month = month


class TestMLBIntegration:
    @mock.patch('requests.get', side_effect=mock_pyquery)
    def setup_method(self, *args, **kwargs):
        self.results = {
            'rank': 3,
            'abbreviation': 'HOU',
            'name': 'Houston Astros',
            'league': 'AL',
            'games': 162,
            'wins': 101,
            'losses': 61,
            'win_percentage': .624,
            'streak': None,
            'runs': 5.5,
            'runs_against': 4.3,
            'run_difference': 1.2,
            'strength_of_schedule': 0.0,
            'simple_rating_system': 1.2,
            'pythagorean_win_loss': '99-63',
            'luck': 2,
            'interleague_record': '15-5',
            'home_record': '48-33',
            'home_wins': 48,
            'home_losses': 33,
            'away_record': '53-28',
            'away_wins': 53,
            'away_losses': 28,
            'extra_inning_record': '4-4',
            'extra_inning_wins': 4,
            'extra_inning_losses': 4,
            'single_run_record': '19-13',
            'single_run_wins': 19,
            'single_run_losses': 13,
            'record_vs_right_handed_pitchers': '80-37',
            'wins_vs_right_handed_pitchers': 80,
            'losses_vs_right_handed_pitchers': 37,
            'record_vs_left_handed_pitchers': '21-24',
            'wins_vs_left_handed_pitchers': 21,
            'losses_vs_left_handed_pitchers': 24,
            'record_vs_teams_over_500': '18-15',
            'wins_vs_teams_over_500': 18,
            'losses_vs_teams_over_500': 15,
            'record_vs_teams_under_500': '83-46',
            'wins_vs_teams_under_500': 83,
            'losses_vs_teams_under_500': 46,
            'last_ten_games_record': None,
            'wins_last_ten_games': None,
            'losses_last_ten_games': None,
            'last_twenty_games_record': None,
            'wins_last_twenty_games': None,
            'losses_last_twenty_games': None,
            'last_thirty_games_record': None,
            'wins_last_thirty_games': None,
            'losses_last_thirty_games': None,
            'number_players_used': 46,
            'average_batter_age': 28.8,
            'plate_appearances': 6271,
            'at_bats': 5611,
            'total_runs': 896,
            'hits': 1581,
            'doubles': 346,
            'triples': 20,
            'home_runs': 238,
            'runs_batted_in': 854,
            'stolen_bases': 98,
            'times_caught_stealing': 42,
            'bases_on_balls': 509,
            'times_struck_out': 1087,
            'batting_average': .282,
            'on_base_percentage': .346,
            'slugging_percentage': .478,
            'on_base_plus_slugging_percentage': .823,
            'on_base_plus_slugging_percentage_plus': 127,
            'total_bases': 2681,
            'grounded_into_double_plays': 139,
            'times_hit_by_pitch': 70,
            'sacrifice_hits': 11,
            'sacrifice_flies': 61,
            'intentional_bases_on_balls': 27,
            'runners_left_on_base': 1094,
            'number_of_pitchers': 27,
            'average_pitcher_age': 28.5,
            'runs_allowed_per_game': 4.32,
            'earned_runs_against': 4.12,
            'games_finished': 161,
            'complete_games': 1,
            'shutouts': 9,
            'complete_game_shutouts': 0,
            'saves': 45,
            'innings_pitched': 1446.0,
            'hits_allowed': 1314,
            'home_runs_against': 192,
            'bases_on_walks_given': 522,
            'strikeouts': 1593,
            'hit_pitcher': 70,
            'balks': 4,
            'wild_pitches': 86,
            'batters_faced': 6111,
            'earned_runs_against_plus': 96,
            'fielding_independent_pitching': 3.91,
            'whip': 1.270,
            'hits_per_nine_innings': 8.2,
            'home_runs_per_nine_innings': 1.2,
            'bases_on_walks_given_per_nine_innings': 3.2,
            'strikeouts_per_nine_innings': 9.9,
            'strikeouts_per_base_on_balls': 3.05,
            'opposing_runners_left_on_base': 1073
        }
        self.abbreviations = [
            'NYY', 'BOS', 'ATL', 'LAA', 'HOU', 'MIL', 'PHI', 'ARI', 'STL',
            'PIT', 'SEA', 'WSN', 'CHC', 'COL', 'NYM', 'TOR', 'CLE', 'SFG',
            'OAK', 'MIN', 'DET', 'TBR', 'LAD', 'TEX', 'SDP', 'MIA', 'CIN',
            'KCR', 'BAL', 'CHW'
        ]

        flexmock(utils) \
            .should_receive('_todays_date') \
            .and_return(MockDateTime(YEAR, MONTH))

        self.teams = Teams()

    def test_mlb_integration_returns_correct_number_of_teams(self):
        assert len(self.teams) == len(self.abbreviations)

    def test_mlb_integration_returns_correct_attributes_for_team(self):
        houston = self.teams('HOU')

        for attribute, value in self.results.items():
            assert getattr(houston, attribute) == value

    def test_mlb_integration_returns_correct_team_abbreviations(self):
        for team in self.teams:
            assert team.abbreviation in self.abbreviations

    def test_mlb_integration_dataframe_returns_dataframe(self):
        df = pd.DataFrame([self.results], index=['HOU'])

        houston = self.teams('HOU')
        # Pandas doesn't natively allow comparisons of DataFrames.
        # Concatenating the two DataFrames (the one generated during the test
        # and the expected one above) and dropping duplicate rows leaves only
        # the rows that are unique between the two frames. This allows a quick
        # check of the DataFrame to see if it is empty - if so, all rows are
        # duplicates, and they are equal.
        frames = [df, houston.dataframe]
        df1 = pd.concat(frames).drop_duplicates(keep=False)

        assert df1.empty

    def test_mlb_integration_all_teams_dataframe_returns_dataframe(self):
        result = self.teams.dataframes.drop_duplicates(keep=False)

        assert len(result) == len(self.abbreviations)
        assert set(result.columns.values) == set(self.results.keys())

    def test_mlb_invalid_team_name_raises_value_error(self):
        with pytest.raises(ValueError):
            self.teams('INVALID_NAME')



from pyquery import PyQuery as pq
import re
from .. import utils
from .constants import CONFERENCE_URL, CONFERENCES_URL
from six.moves.urllib.error import HTTPError


class Conference:
    """
    Find teams that participated in a particular conference.

    Create a dictionary which includes the names and abbreviations for all
    teams that participated in a conference during a given year.

    Parameters
    ----------
    conference_abbreviation : string
        A string of the requested conference's abbreviation, such as
        'big-12'.
    year : string (optional)
        A string of the requested year to pull conference information from.
        Defaults to the most recent season.
    """
    def __init__(self, conference_abbreviation, year=None):
        self._teams = {}

        self._find_conference_teams(conference_abbreviation, year)

    def _pull_conference_page(self, conference_abbreviation, year):
        """
        Download the conference page.

        Download the conference page for the requested conference and season
        and create a PyQuery object.

        Parameters
        ----------
        conference_abbreviation : string
            A string of the requested conference's abbreviation, such as
            'big-12'.
        year : string
            A string of the requested year to pull conference information from.
        """
        try:
            return pq(CONFERENCE_URL % (conference_abbreviation, year))
        except HTTPError:
            return None

    def _get_team_abbreviation(self, team):
        """
        Retrieve team's abbreviation.

        The team's abbreviation is embedded within the 'school_name' tag and
        requires special parsing as it is located in the middle of a URI. The
        abbreviation is returned for the requested school.

        Parameters
        ----------
        team : PyQuery object
            A PyQuery object representing a single row in a table on the
            conference page.

        Returns
        -------
        string
            Returns a string of the team's abbreviation, such as 'PURDUE'.
        """
        name_tag = team('td[data-stat="school_name"] a')
        team_abbreviation = re.sub(r'.*/cbb/schools/', '', str(name_tag))
        team_abbreviation = re.sub(r'/.*', '', team_abbreviation)
        return team_abbreviation

    def _find_conference_teams(self, conference_abbreviation, year):
        """
        Retrieve the teams in the conference for the requested season.

        Find and retrieve all teams that participated in a conference for a
        given season. The name and abbreviation for each team are parsed and
        recorded to enable easy queries of conference schools..

        Parameters
        ----------
        conference_abbreviation : string
            A string of the requested conference's abbreviation, such as
            'big-12'.
        year : string
            A string of the requested year to pull conference information from.
        """
        if not year:
            year = utils._find_year_for_season('ncaab')
        page = self._pull_conference_page(conference_abbreviation, year)
        if not page:
            url = CONFERENCE_URL % (conference_abbreviation, year)
            output = ("Can't pull requested conference page. Ensure the "
                      "following URL exists: %s" % url)
            raise ValueError(output)
        conference = page('table#standings tbody tr').items()
        for team in conference:
            team_abbreviation = self._get_team_abbreviation(team)
            if team_abbreviation == '':
                continue
            team_name = team('td[data-stat="school_name"]').text()
            self._teams[team_abbreviation] = team_name

    @property
    def teams(self):
        """
        Returns a ``dictionary`` of team names and abbreviations where each key
        is a ``string`` of the team abbreviation and each value is a ``string``
        of the full team name.
        """
        return self._teams


class Conferences:
    """
    Get all conferences and teams for a season.

    Retrieve a list of all conferences and teams that participated in the
    conference for each team in the season. The included properties allow
    flexibility in queries to either get the conference abbreviation for a
    given team, or get more detailed information including all teams for each
    conference.

    Parameters
    ----------
    year : string (optional)
        A string of the requested year to pull conferences from. Defaults to
        the most recent season.
    """
    def __init__(self, year=None):
        self._conferences = {}
        self._team_conference = {}

        self._find_conferences(year)

    def _pull_conference_page(self, year):
        """
        Download the conference page.

        Download the conference page for the requested team and create a
        PyQuery object.

        Parameters
        ----------
        year : string
            A string of the requested year to pull rankings from.

        Returns
        -------
        PyQuery object
            Returns a PyQuery object of the conference HTML page.
        """
        try:
            return pq(CONFERENCES_URL % year)
        except HTTPError:
            return None

    def _get_conference_id(self, conference):
        """
        Get the conference abbreviation, such as 'big-12'.

        The conference abbreviation is embedded within the Conference Name
        tag and requires special parsing to extract. The abbreviation is
        returned as a string.

        Parameters
        ----------
        conference : PyQuery object
            A PyQuery object representing a single row in the conference table
            which can be used to find the conference abbreviation.

        Returns
        -------
        string
            Returns a string of the conference abbreviation, such as 'big-12'.
        """
        name_tag = conference('td[data-stat="conf_name"] a')
        conference_id = re.sub(r'.*/cbb/conferences/', '', str(name_tag))
        conference_id = re.sub(r'/.*', '', conference_id)
        return conference_id

    def _find_conferences(self, year):
        """
        Retrieve the conferences and teams for the requested season.

        Find and retrieve all conferences for a given season and parse all of
        the teams that participated in the conference during that year.
        Conference information includes abbreviation and full name for the
        conference as well as the abbreviation and full name for each team in
        the conference.

        Parameters
        ----------
        year : string
            A string of the requested year to pull conferences from.
        """
        if not year:
            year = utils._find_year_for_season('ncaab')
        page = self._pull_conference_page(year)
        if not page:
            output = ("Can't pull requested conference page. Ensure the "
                      "following URL exists: %s" % (CONFERENCES_URL % year))
            raise ValueError(output)
        conferences = page('table#conference-summary tbody tr').items()
        for conference in conferences:
            conference_abbreviation = self._get_conference_id(conference)
            conference_name = conference('td[data-stat="conf_name"]').text()
            teams_dict = Conference(conference_abbreviation, year).teams
            conference_dict = {
                    'name': conference_name,
                    'teams': teams_dict
                }
            for team in teams_dict.keys():
                self._team_conference[team] = conference_abbreviation
            self._conferences[conference_abbreviation] = conference_dict

    @property
    def conferences(self):
        """
        Returns a ``dictionary`` of conference names and abbreviations where
        each key is a ``string`` of the abbreviation and each value is a
        ``dictionary`` containing the full conference name and another
        ``dictionary`` with individual team information. The overall dictionary
        is in the following structure::

            {
                abbreviation, ie 'big-12' (str): {
                    'name': Full conference name, such as 'Big 12 Conference'
                            (str),
                    'teams': {
                        team abbreviation, such as 'kansas' (str): Full team
                            name, such as 'Kansas' (str),
                        ...
                    }
                },
                ...
            }
        """
        return self._conferences

    @property
    def team_conference(self):
        """
        Returns a ``dictionary`` of conference abbreviations for each team
        where each key is a ``string`` of the team abbreviation and each value
        is a ``string`` of the conference abbreviation.
        """
        return self._team_conference



import mock
import os
import pandas as pd
import pytest
from datetime import datetime
from flexmock import flexmock
from sportsreference import utils
from sportsreference.nba.roster import Player, Roster
from sportsreference.nba.teams import Team


def read_file(filename):
    filepath = os.path.join(os.path.dirname(__file__), 'nba', filename)
    return open('%s.html' % filepath, 'r').read()


def mock_pyquery(url):
    class MockPQ:
        def __init__(self, html_contents, status=200):
            self.url = url
            self.reason = 'Bad URL'  # Used when throwing HTTPErrors
            self.headers = {}  # Used when throwing HTTPErrors
            self.status_code = status
            self.html_contents = html_contents
            self.text = html_contents

    if 'HOU' in url:
        return MockPQ(read_file('2018'))
    if 'anderry01' in url:
        return MockPQ(read_file('anderry01'))
    if 'arizatr01' in url:
        return MockPQ(read_file('arizatr01'))
    if 'blackta01' in url:
        return MockPQ(read_file('blackta01'))
    if 'youngtr01' in url:
        return MockPQ(read_file('youngtr01'))
    if 'BAD' in url:
        return MockPQ(None, 404)
    return MockPQ(read_file('hardeja01'))


class TestNBAPlayer:
    @mock.patch('requests.get', side_effect=mock_pyquery)
    def setup_method(self, *args, **kwargs):
        self.results_career = {
            'player_id': 'hardeja01',
            'season': 'Career',
            'name': 'James Harden',
            'team_abbreviation': '',
            'position': 'SG',
            'height': '6-5',
            'weight': 220,
            'birth_date': datetime(1989, 8, 26),
            'nationality': 'United States of America',
            'games_played': 687,
            'games_started': 474,
            'minutes_played': 23239,
            'field_goals': 4656,
            'field_goal_attempts': 10502,
            'field_goal_percentage': .443,
            'three_pointers': 1647,
            'three_point_attempts': 4523,
            'three_point_percentage': .364,
            'two_pointers': 3009,
            'two_point_attempts': 5979,
            'two_point_percentage': .503,
            'effective_field_goal_percentage': .522,
            'free_throws': 4850,
            'free_throw_attempts': 5678,
            'free_throw_percentage': .854,
            'offensive_rebounds': 516,
            'defensive_rebounds': 2966,
            'total_rebounds': 3482,
            'assists': 4157,
            'steals': 1031,
            'blocks': 325,
            'turnovers': 2383,
            'personal_fouls': 1733,
            'points': 15809,
            'player_efficiency_rating': 23.7,
            'true_shooting_percentage': .608,
            'three_point_attempt_rate': .431,
            'free_throw_attempt_rate': .541,
            'offensive_rebound_percentage': 2.5,
            'defensive_rebound_percentage': 14.1,
            'total_rebound_percentage': 8.4,
            'assist_percentage': 30.9,
            'steal_percentage': 2.2,
            'block_percentage': 1.1,
            'turnover_percentage': 15.5,
            'usage_percentage': 28.9,
            'offensive_win_shares': 80.5,
            'defensive_win_shares': 26.2,
            'win_shares': 106.7,
            'win_shares_per_48_minutes': .220,
            'offensive_box_plus_minus': 6.3,
            'defensive_box_plus_minus': 0.2,
            'box_plus_minus': 6.5,
            'value_over_replacement_player': 50.0,
            'shooting_distance': 14.7,
            'percentage_shots_two_pointers': .569,
            'percentage_zero_to_three_footers': .282,
            'percentage_three_to_ten_footers': .117,
            'percentage_ten_to_sixteen_footers': .068,
            'percentage_sixteen_foot_plus_two_pointers': .103,
            'percentage_shots_three_pointers': .431,
            'field_goal_perc_zero_to_three_feet': .640,
            'field_goal_perc_three_to_ten_feet': .331,
            'field_goal_perc_ten_to_sixteen_feet': .421,
            'field_goal_perc_sixteen_foot_plus_two_pointers': .379,
            'two_pointers_assisted_percentage': .199,
            'percentage_field_goals_as_dunks': .029,
            'dunks': 290,
            'three_pointers_assisted_percentage': .514,
            'percentage_of_three_pointers_from_corner': .089,
            'three_point_shot_percentage_from_corner': .375,
            'half_court_heaves': 14,
            'half_court_heaves_made': 1,
            'point_guard_percentage': 15,
            'shooting_guard_percentage': 72,
            'small_forward_percentage': 13,
            'power_forward_percentage': 0,
            'center_percentage': 0,
            'on_court_plus_minus': 6.0,
            'net_plus_minus': 4.8,
            'passing_turnovers': 1236,
            'lost_ball_turnovers': 740,
            'other_turnovers': 406,
            'shooting_fouls': 668,
            'blocking_fouls': 25,
            'offensive_fouls': 275,
            'take_fouls': 27,
            'points_generated_by_assists': 9910,
            'shooting_fouls_drawn': 2223,
            'and_ones': 366,
            'shots_blocked': 693,
            'salary': 117809888,
            'contract': {
                '2018-19': '$30,431,854',
                '2019-20': '$37,800,000',
                '2020-21': '$40,824,000',
                '2021-22': '$43,848,000',
                '2022-23': '$46,872,000'
            }
        }

        self.results_2018 = {
            'player_id': 'hardeja01',
            'season': '2017-18',
            'name': 'James Harden',
            'team_abbreviation': 'HOU',
            'position': 'SG',
            'height': '6-5',
            'weight': 220,
            'birth_date': datetime(1989, 8, 26),
            'nationality': 'United States of America',
            'games_played': 72,
            'games_started': 72,
            'minutes_played': 2551,
            'field_goals': 651,
            'field_goal_attempts': 1449,
            'field_goal_percentage': .449,
            'three_pointers': 265,
            'three_point_attempts': 722,
            'three_point_percentage': .367,
            'two_pointers': 386,
            'two_point_attempts': 727,
            'two_point_percentage': .531,
            'effective_field_goal_percentage': .541,
            'free_throws': 624,
            'free_throw_attempts': 727,
            'free_throw_percentage': .858,
            'offensive_rebounds': 41,
            'defensive_rebounds': 348,
            'total_rebounds': 389,
            'assists': 630,
            'steals': 126,
            'blocks': 50,
            'turnovers': 315,
            'personal_fouls': 169,
            'points': 2191,
            'player_efficiency_rating': 29.8,
            'true_shooting_percentage': .619,
            'three_point_attempt_rate': .498,
            'free_throw_attempt_rate': .502,
            'offensive_rebound_percentage': 1.8,
            'defensive_rebound_percentage': 15.2,
            'total_rebound_percentage': 8.6,
            'assist_percentage': 45.1,
            'steal_percentage': 2.4,
            'block_percentage': 1.7,
            'turnover_percentage': 15.1,
            'usage_percentage': 36.1,
            'offensive_win_shares': 11.6,
            'defensive_win_shares': 3.8,
            'win_shares': 15.4,
            'win_shares_per_48_minutes': .289,
            'offensive_box_plus_minus': 9.6,
            'defensive_box_plus_minus': 1.3,
            'box_plus_minus': 10.9,
            'value_over_replacement_player': 8.3,
            'shooting_distance': 15.3,
            'percentage_shots_two_pointers': .502,
            'percentage_zero_to_three_footers': .273,
            'percentage_three_to_ten_footers': .118,
            'percentage_ten_to_sixteen_footers': .071,
            'percentage_sixteen_foot_plus_two_pointers': .039,
            'percentage_shots_three_pointers': .498,
            'field_goal_perc_zero_to_three_feet': .658,
            'field_goal_perc_three_to_ten_feet': .351,
            'field_goal_perc_ten_to_sixteen_feet': .437,
            'field_goal_perc_sixteen_foot_plus_two_pointers': .368,
            'two_pointers_assisted_percentage': .088,
            'percentage_field_goals_as_dunks': .013,
            'dunks': 19,
            'three_pointers_assisted_percentage': .260,
            'percentage_of_three_pointers_from_corner': .072,
            'three_point_shot_percentage_from_corner': .327,
            'half_court_heaves': 2,
            'half_court_heaves_made': 0,
            'point_guard_percentage': 19,
            'shooting_guard_percentage': 75,
            'small_forward_percentage': 6,
            'power_forward_percentage': 0,
            'center_percentage': 0,
            'on_court_plus_minus': 10.5,
            'net_plus_minus': 5.3,
            'passing_turnovers': 188,
            'lost_ball_turnovers': 87,
            'other_turnovers': 40,
            'shooting_fouls': 65,
            'blocking_fouls': 2,
            'offensive_fouls': 29,
            'take_fouls': 5,
            'points_generated_by_assists': 1499,
            'shooting_fouls_drawn': 273,
            'and_ones': 55,
            'shots_blocked': 98,
            'salary': 28299399,
            'contract': {
                '2018-19': '$30,431,854',
                '2019-20': '$37,800,000',
                '2020-21': '$40,824,000',
                '2021-22': '$43,848,000',
                '2022-23': '$46,872,000'
            }
        }

        self.player = Player('hardeja01')

    def test_nba_player_returns_requested_player_career_stats(self):
        # Request the career stats
        player = self.player('')

        for attribute, value in self.results_career.items():
            assert getattr(player, attribute) == value

    def test_nba_player_returns_requested_player_season_stats(self):
        # Request the 2017-18 stats
        player = self.player('2017-18')

        for attribute, value in self.results_2018.items():
            assert getattr(player, attribute) == value

    def test_dataframe_returns_dataframe(self):
        dataframe = [
            {'field_goal_perc_ten_to_sixteen_feet': 0.463,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.411,
             'box_plus_minus': 5.7,
             'two_point_percentage': 0.515,
             'two_pointers': 372,
             'net_plus_minus': 7.6,
             'personal_fouls': 177,
             'three_point_percentage': 0.366,
             'player_efficiency_rating': 23.5,
             'offensive_rebound_percentage': 2.6,
             'percentage_sixteen_foot_plus_two_pointers': 0.125,
             'three_point_attempt_rate': 0.401,
             'field_goals': 549,
             'free_throw_attempts': 665,
             'shooting_fouls_drawn': 261,
             'defensive_rebound_percentage': 10.7,
             'steal_percentage': 2.1,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.395,
             'percentage_ten_to_sixteen_footers': 0.079,
             'turnover_percentage': 15.0,
             'points_generated_by_assists': 1040,
             'block_percentage': 0.8,
             'field_goal_percentage': 0.456,
             'percentage_of_three_pointers_from_corner': 0.066,
             'free_throw_percentage': 0.866,
             'offensive_fouls': 32,
             'half_court_heaves': 1,
             'percentage_three_to_ten_footers': 0.122,
             'passing_turnovers': 135,
             'value_over_replacement_player': 5.4,
             'blocking_fouls': 9,
             'field_goal_attempts': 1205,
             'free_throw_attempt_rate': 0.552,
             'effective_field_goal_percentage': 0.529,
             'total_rebounds': 344,
             'center_percentage': 0,
             'power_forward_percentage': 0,
             'nationality': 'United States of America',
             'offensive_win_shares': 10.1,
             'defensive_rebounds': 283,
             'total_rebound_percentage': 6.9,
             'assists': 446,
             'defensive_box_plus_minus': -0.5,
             'shooting_distance': 14.8,
             'height': '6-5',
             'true_shooting_percentage': 0.618,
             'small_forward_percentage': 7,
             'minutes_played': 2777,
             'on_court_plus_minus': 7.1,
             'games_started': 73,
             'win_shares_per_48_minutes': 0.221,
             'dunks': 45,
             'three_pointers_assisted_percentage': 0.588,
             'two_point_attempts': 722,
             'salary': 13701250,
             'offensive_box_plus_minus': 6.2,
             'lost_ball_turnovers': 79,
             'percentage_shots_two_pointers': 0.599,
             'shooting_guard_percentage': 92,
             'shots_blocked': 61,
             'and_ones': 42,
             'three_pointers': 177,
             'take_fouls': 1,
             'free_throws': 576,
             'usage_percentage': 27.8,
             'assist_percentage': 27.3,
             'three_point_attempts': 483,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.401,
             'percentage_field_goals_as_dunks': 0.039,
             'offensive_rebounds': 61,
             'other_turnovers': 51,
             'point_guard_percentage': 1,
             'half_court_heaves_made': 0,
             'blocks': 29,
             'two_pointers_assisted_percentage': 0.223,
             'win_shares': 12.8,
             'defensive_win_shares': 2.7,
             'games_played': 73,
             'percentage_zero_to_three_footers': 0.273,
             'steals': 115,
             'turnovers': 265,
             'field_goal_perc_zero_to_three_feet': 0.632,
             'shooting_fouls': 51,
             'team_abbreviation': 'HOU',
             'points': 1851,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.375},
            {'field_goal_perc_ten_to_sixteen_feet': 0.421,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.379,
             'box_plus_minus': 6.5,
             'two_point_percentage': 0.503,
             'two_pointers': 3009,
             'net_plus_minus': 4.8,
             'personal_fouls': 1733,
             'three_point_percentage': 0.364,
             'player_efficiency_rating': 23.7,
             'offensive_rebound_percentage': 2.5,
             'percentage_sixteen_foot_plus_two_pointers': 0.103,
             'three_point_attempt_rate': 0.431,
             'field_goals': 4656,
             'free_throw_attempts': 5678,
             'shooting_fouls_drawn': 2223,
             'defensive_rebound_percentage': 14.1,
             'steal_percentage': 2.2,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.331,
             'percentage_ten_to_sixteen_footers': 0.068,
             'turnover_percentage': 15.5,
             'points_generated_by_assists': 9910,
             'block_percentage': 1.1,
             'field_goal_percentage': 0.443,
             'percentage_of_three_pointers_from_corner': 0.089,
             'free_throw_percentage': 0.854,
             'offensive_fouls': 275,
             'half_court_heaves': 14,
             'percentage_three_to_ten_footers': 0.117,
             'passing_turnovers': 1236,
             'value_over_replacement_player': 50.0,
             'blocking_fouls': 25,
             'field_goal_attempts': 10502,
             'free_throw_attempt_rate': 0.541,
             'effective_field_goal_percentage': 0.522,
             'total_rebounds': 3482,
             'center_percentage': 0,
             'power_forward_percentage': 0,
             'nationality': 'United States of America',
             'offensive_win_shares': 80.5,
             'defensive_rebounds': 2966,
             'total_rebound_percentage': 8.4,
             'assists': 4157,
             'defensive_box_plus_minus': 0.2,
             'shooting_distance': 14.7,
             'height': '6-5',
             'true_shooting_percentage': 0.608,
             'small_forward_percentage': 13,
             'minutes_played': 23239,
             'on_court_plus_minus': 6.0,
             'games_started': 474,
             'win_shares_per_48_minutes': 0.22,
             'dunks': 290,
             'three_pointers_assisted_percentage': 0.514,
             'two_point_attempts': 5979,
             'salary': 117809888,
             'offensive_box_plus_minus': 6.3,
             'lost_ball_turnovers': 740,
             'percentage_shots_two_pointers': 0.569,
             'shooting_guard_percentage': 72,
             'shots_blocked': 693,
             'and_ones': 366,
             'three_pointers': 1647,
             'take_fouls': 27,
             'free_throws': 4850,
             'usage_percentage': 28.9,
             'assist_percentage': 30.9,
             'three_point_attempts': 4523,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.431,
             'percentage_field_goals_as_dunks': 0.029,
             'offensive_rebounds': 516,
             'other_turnovers': 406,
             'point_guard_percentage': 15,
             'half_court_heaves_made': 1,
             'blocks': 325,
             'two_pointers_assisted_percentage': 0.199,
             'win_shares': 106.7,
             'defensive_win_shares': 26.2,
             'games_played': 687,
             'percentage_zero_to_three_footers': 0.282,
             'steals': 1031,
             'turnovers': 2383,
             'field_goal_perc_zero_to_three_feet': 0.64,
             'shooting_fouls': 668,
             'team_abbreviation': 'HOU',
             'points': 15809,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.375},
            {'field_goal_perc_ten_to_sixteen_feet': 0.431,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.337,
             'box_plus_minus': 8.4,
             'two_point_percentage': 0.48,
             'two_pointers': 439,
             'net_plus_minus': 8.8,
             'personal_fouls': 208,
             'three_point_percentage': 0.375,
             'player_efficiency_rating': 26.7,
             'offensive_rebound_percentage': 2.8,
             'percentage_sixteen_foot_plus_two_pointers': 0.131,
             'three_point_attempt_rate': 0.378,
             'field_goals': 647,
             'free_throw_attempts': 824,
             'shooting_fouls_drawn': 318,
             'defensive_rebound_percentage': 14.2,
             'steal_percentage': 2.6,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.291,
             'percentage_ten_to_sixteen_footers': 0.074,
             'turnover_percentage': 14.9,
             'points_generated_by_assists': 1382,
             'block_percentage': 1.6,
             'field_goal_percentage': 0.44,
             'percentage_of_three_pointers_from_corner': 0.108,
             'free_throw_percentage': 0.868,
             'offensive_fouls': 36,
             'half_court_heaves': 1,
             'percentage_three_to_ten_footers': 0.107,
             'passing_turnovers': 176,
             'value_over_replacement_player': 7.8,
             'blocking_fouls': 4,
             'field_goal_attempts': 1470,
             'free_throw_attempt_rate': 0.561,
             'effective_field_goal_percentage': 0.511,
             'total_rebounds': 459,
             'center_percentage': 0,
             'power_forward_percentage': 0,
             'nationality': 'United States of America',
             'offensive_win_shares': 12.2,
             'defensive_rebounds': 384,
             'total_rebound_percentage': 8.5,
             'assists': 565,
             'defensive_box_plus_minus': 1.0,
             'shooting_distance': 14.0,
             'height': '6-5',
             'true_shooting_percentage': 0.605,
             'small_forward_percentage': 7,
             'minutes_played': 2981,
             'on_court_plus_minus': 5.8,
             'games_started': 81,
             'win_shares_per_48_minutes': 0.265,
             'dunks': 33,
             'three_pointers_assisted_percentage': 0.51,
             'two_point_attempts': 915,
             'salary': 14728844,
             'offensive_box_plus_minus': 7.4,
             'lost_ball_turnovers': 97,
             'percentage_shots_two_pointers': 0.622,
             'shooting_guard_percentage': 92,
             'shots_blocked': 107,
             'and_ones': 50,
             'three_pointers': 208,
             'take_fouls': 3,
             'free_throws': 715,
             'usage_percentage': 31.3,
             'assist_percentage': 34.6,
             'three_point_attempts': 555,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.378,
             'percentage_field_goals_as_dunks': 0.023,
             'offensive_rebounds': 75,
             'other_turnovers': 48,
             'point_guard_percentage': 1,
             'half_court_heaves_made': 0,
             'blocks': 60,
             'two_pointers_assisted_percentage': 0.166,
             'win_shares': 16.4,
             'defensive_win_shares': 4.2,
             'games_played': 81,
             'percentage_zero_to_three_footers': 0.31,
             'steals': 154,
             'turnovers': 321,
             'field_goal_perc_zero_to_three_feet': 0.618,
             'shooting_fouls': 84,
             'team_abbreviation': 'HOU',
             'points': 2217,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.467},
            {'field_goal_perc_ten_to_sixteen_feet': 0.419,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.427,
             'box_plus_minus': 10.1,
             'two_point_percentage': 0.53,
             'two_pointers': 412,
             'net_plus_minus': 3.0,
             'personal_fouls': 215,
             'three_point_percentage': 0.347,
             'player_efficiency_rating': 27.4,
             'offensive_rebound_percentage': 3.5,
             'percentage_sixteen_foot_plus_two_pointers': 0.053,
             'three_point_attempt_rate': 0.493,
             'field_goals': 674,
             'free_throw_attempts': 881,
             'shooting_fouls_drawn': 305,
             'defensive_rebound_percentage': 20.9,
             'steal_percentage': 2.0,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.364,
             'percentage_ten_to_sixteen_footers': 0.076,
             'turnover_percentage': 19.5,
             'points_generated_by_assists': 2179,
             'block_percentage': 1.0,
             'field_goal_percentage': 0.44,
             'percentage_of_three_pointers_from_corner': 0.056,
             'free_throw_percentage': 0.847,
             'offensive_fouls': 34,
             'half_court_heaves': 3,
             'percentage_three_to_ten_footers': 0.134,
             'passing_turnovers': 294,
             'value_over_replacement_player': 9.0,
             'blocking_fouls': 6,
             'field_goal_attempts': 1533,
             'free_throw_attempt_rate': 0.575,
             'effective_field_goal_percentage': 0.525,
             'total_rebounds': 659,
             'center_percentage': 0,
             'power_forward_percentage': 0,
             'nationality': 'United States of America',
             'offensive_win_shares': 11.5,
             'defensive_rebounds': 564,
             'total_rebound_percentage': 12.2,
             'assists': 907,
             'defensive_box_plus_minus': 1.5,
             'shooting_distance': 15.9,
             'height': '6-5',
             'true_shooting_percentage': 0.613,
             'small_forward_percentage': 0,
             'minutes_played': 2947,
             'on_court_plus_minus': 6.9,
             'games_started': 81,
             'win_shares_per_48_minutes': 0.245,
             'dunks': 21,
             'three_pointers_assisted_percentage': 0.317,
             'two_point_attempts': 777,
             'salary': 26540100,
             'offensive_box_plus_minus': 8.7,
             'lost_ball_turnovers': 114,
             'percentage_shots_two_pointers': 0.507,
             'shooting_guard_percentage': 1,
             'shots_blocked': 85,
             'and_ones': 29,
             'three_pointers': 262,
             'take_fouls': 2,
             'free_throws': 746,
             'usage_percentage': 34.2,
             'assist_percentage': 50.7,
             'three_point_attempts': 756,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.493,
             'percentage_field_goals_as_dunks': 0.014,
             'offensive_rebounds': 95,
             'other_turnovers': 55,
             'point_guard_percentage': 98,
             'half_court_heaves_made': 0,
             'blocks': 38,
             'two_pointers_assisted_percentage': 0.095,
             'win_shares': 15.0,
             'defensive_win_shares': 3.6,
             'games_played': 81,
             'percentage_zero_to_three_footers': 0.242,
             'steals': 121,
             'turnovers': 464,
             'field_goal_perc_zero_to_three_feet': 0.682,
             'shooting_fouls': 80,
             'team_abbreviation': 'HOU',
             'points': 2356,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.357},
            {'field_goal_perc_ten_to_sixteen_feet': 0.368,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.381,
             'box_plus_minus': 1.0,
             'two_point_percentage': 0.424,
             'two_pointers': 140,
             'net_plus_minus': 3.8,
             'personal_fouls': 200,
             'three_point_percentage': 0.375,
             'player_efficiency_rating': 14.0,
             'offensive_rebound_percentage': 3.2,
             'percentage_sixteen_foot_plus_two_pointers': 0.168,
             'three_point_attempt_rate': 0.429,
             'field_goals': 233,
             'free_throw_attempts': 240,
             'shooting_fouls_drawn': 113,
             'defensive_rebound_percentage': 12.7,
             'steal_percentage': 2.4,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.308,
             'percentage_ten_to_sixteen_footers': 0.033,
             'turnover_percentage': 13.4,
             'points_generated_by_assists': 299,
             'block_percentage': 0.9,
             'field_goal_percentage': 0.403,
             'percentage_of_three_pointers_from_corner': 0.19,
             'free_throw_percentage': 0.808,
             'offensive_fouls': 12,
             'half_court_heaves': 2,
             'percentage_three_to_ten_footers': 0.045,
             'passing_turnovers': 40,
             'value_over_replacement_player': 1.3,
             'blocking_fouls': 0,
             'field_goal_attempts': 578,
             'free_throw_attempt_rate': 0.415,
             'effective_field_goal_percentage': 0.484,
             'total_rebounds': 244,
             'center_percentage': 0,
             'power_forward_percentage': 0,
             'nationality': 'United States of America',
             'offensive_win_shares': 2.0,
             'defensive_rebounds': 197,
             'total_rebound_percentage': 8.1,
             'assists': 137,
             'defensive_box_plus_minus': 0.3,
             'shooting_distance': 14.7,
             'height': '6-5',
             'true_shooting_percentage': 0.551,
             'small_forward_percentage': 9,
             'minutes_played': 1738,
             'on_court_plus_minus': 6.4,
             'games_started': 0,
             'win_shares_per_48_minutes': 0.124,
             'dunks': 25,
             'three_pointers_assisted_percentage': 0.925,
             'two_point_attempts': 330,
             'salary': 4054160,
             'offensive_box_plus_minus': 0.7,
             'lost_ball_turnovers': 36,
             'percentage_shots_two_pointers': 0.571,
             'shooting_guard_percentage': 90,
             'shots_blocked': 45,
             'and_ones': 14,
             'three_pointers': 93,
             'take_fouls': 0,
             'free_throws': 194,
             'usage_percentage': 20.4,
             'assist_percentage': 12.3,
             'three_point_attempts': 248,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.429,
             'percentage_field_goals_as_dunks': 0.048,
             'offensive_rebounds': 47,
             'other_turnovers': 30,
             'point_guard_percentage': 1,
             'half_court_heaves_made': 0,
             'blocks': 20,
             'two_pointers_assisted_percentage': 0.379,
             'win_shares': 4.5,
             'defensive_win_shares': 2.5,
             'games_played': 76,
             'percentage_zero_to_three_footers': 0.325,
             'steals': 80,
             'turnovers': 106,
             'field_goal_perc_zero_to_three_feet': 0.468,
             'shooting_fouls': 98,
             'team_abbreviation': 'HOU',
             'points': 753,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.468},
            {'field_goal_perc_ten_to_sixteen_feet': 0.426,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.402,
             'box_plus_minus': 6.7,
             'two_point_percentage': 0.494,
             'two_pointers': 474,
             'net_plus_minus': 7.6,
             'personal_fouls': 229,
             'three_point_percentage': 0.359,
             'player_efficiency_rating': 25.3,
             'offensive_rebound_percentage': 2.2,
             'percentage_sixteen_foot_plus_two_pointers': 0.12,
             'three_point_attempt_rate': 0.406,
             'field_goals': 710,
             'free_throw_attempts': 837,
             'shooting_fouls_drawn': 311,
             'defensive_rebound_percentage': 15.6,
             'steal_percentage': 2.2,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.328,
             'percentage_ten_to_sixteen_footers': 0.105,
             'turnover_percentage': 15.9,
             'points_generated_by_assists': 1458,
             'block_percentage': 1.4,
             'field_goal_percentage': 0.439,
             'percentage_of_three_pointers_from_corner': 0.094,
             'free_throw_percentage': 0.86,
             'offensive_fouls': 42,
             'half_court_heaves': 0,
             'percentage_three_to_ten_footers': 0.124,
             'passing_turnovers': 169,
             'value_over_replacement_player': 6.9,
             'blocking_fouls': 4,
             'field_goal_attempts': 1617,
             'free_throw_attempt_rate': 0.518,
             'effective_field_goal_percentage': 0.512,
             'total_rebounds': 501,
             'center_percentage': 0,
             'power_forward_percentage': 1,
             'nationality': 'United States of America',
             'offensive_win_shares': 10.7,
             'defensive_rebounds': 438,
             'total_rebound_percentage': 8.8,
             'assists': 612,
             'defensive_box_plus_minus': -0.4,
             'shooting_distance': 14.4,
             'height': '6-5',
             'true_shooting_percentage': 0.598,
             'small_forward_percentage': 21,
             'minutes_played': 3125,
             'on_court_plus_minus': 1.7,
             'games_started': 82,
             'win_shares_per_48_minutes': 0.204,
             'dunks': 16,
             'three_pointers_assisted_percentage': 0.521,
             'two_point_attempts': 960,
             'salary': 15756438,
             'offensive_box_plus_minus': 7.1,
             'lost_ball_turnovers': 149,
             'percentage_shots_two_pointers': 0.594,
             'shooting_guard_percentage': 77,
             'shots_blocked': 95,
             'and_ones': 56,
             'three_pointers': 236,
             'take_fouls': 11,
             'free_throws': 720,
             'usage_percentage': 32.5,
             'assist_percentage': 35.4,
             'three_point_attempts': 657,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.406,
             'percentage_field_goals_as_dunks': 0.011,
             'offensive_rebounds': 63,
             'other_turnovers': 56,
             'point_guard_percentage': 2,
             'half_court_heaves_made': 0,
             'blocks': 51,
             'two_pointers_assisted_percentage': 0.181,
             'win_shares': 13.3,
             'defensive_win_shares': 2.6,
             'games_played': 82,
             'percentage_zero_to_three_footers': 0.246,
             'steals': 139,
             'turnovers': 374,
             'field_goal_perc_zero_to_three_feet': 0.65,
             'shooting_fouls': 93,
             'team_abbreviation': 'HOU',
             'points': 2376,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.274},
            {'field_goal_perc_ten_to_sixteen_feet': 0.4,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.413,
             'box_plus_minus': 4.5,
             'two_point_percentage': 0.579,
             'two_pointers': 195,
             'net_plus_minus': 8.0,
             'personal_fouls': 150,
             'three_point_percentage': 0.39,
             'player_efficiency_rating': 21.1,
             'offensive_rebound_percentage': 1.9,
             'percentage_sixteen_foot_plus_two_pointers': 0.073,
             'three_point_attempt_rate': 0.464,
             'field_goals': 309,
             'free_throw_attempts': 369,
             'shooting_fouls_drawn': 161,
             'defensive_rebound_percentage': 12.2,
             'steal_percentage': 1.6,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.3,
             'percentage_ten_to_sixteen_footers': 0.024,
             'turnover_percentage': 14.8,
             'points_generated_by_assists': 536,
             'block_percentage': 0.6,
             'field_goal_percentage': 0.491,
             'percentage_of_three_pointers_from_corner': 0.113,
             'free_throw_percentage': 0.846,
             'offensive_fouls': 24,
             'half_court_heaves': 1,
             'percentage_three_to_ten_footers': 0.111,
             'passing_turnovers': 62,
             'value_over_replacement_player': 3.2,
             'blocking_fouls': 0,
             'field_goal_attempts': 629,
             'free_throw_attempt_rate': 0.587,
             'effective_field_goal_percentage': 0.582,
             'total_rebounds': 252,
             'center_percentage': 0,
             'power_forward_percentage': 0,
             'nationality': 'United States of America',
             'offensive_win_shares': 7.5,
             'defensive_rebounds': 222,
             'total_rebound_percentage': 7.4,
             'assists': 229,
             'defensive_box_plus_minus': -1.1,
             'shooting_distance': 14.3,
             'height': '6-5',
             'true_shooting_percentage': 0.66,
             'small_forward_percentage': 44,
             'minutes_played': 1946,
             'on_court_plus_minus': 9.9,
             'games_started': 2,
             'win_shares_per_48_minutes': 0.23,
             'dunks': 39,
             'three_pointers_assisted_percentage': 0.86,
             'two_point_attempts': 337,
             'salary': 4604760,
             'offensive_box_plus_minus': 5.6,
             'lost_ball_turnovers': 39,
             'percentage_shots_two_pointers': 0.536,
             'shooting_guard_percentage': 55,
             'shots_blocked': 50,
             'and_ones': 28,
             'three_pointers': 114,
             'take_fouls': 0,
             'free_throws': 312,
             'usage_percentage': 21.6,
             'assist_percentage': 19.3,
             'three_point_attempts': 292,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.464,
             'percentage_field_goals_as_dunks': 0.062,
             'offensive_rebounds': 30,
             'other_turnovers': 36,
             'point_guard_percentage': 0,
             'half_court_heaves_made': 0,
             'blocks': 15,
             'two_pointers_assisted_percentage': 0.282,
             'win_shares': 9.3,
             'defensive_win_shares': 1.8,
             'games_played': 62,
             'percentage_zero_to_three_footers': 0.328,
             'steals': 62,
             'turnovers': 137,
             'field_goal_perc_zero_to_three_feet': 0.723,
             'shooting_fouls': 49,
             'team_abbreviation': 'HOU',
             'points': 1044,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.455},
            {'field_goal_perc_ten_to_sixteen_feet': 0.318,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.333,
             'box_plus_minus': 5.5,
             'two_point_percentage': 0.477,
             'two_pointers': 406,
             'net_plus_minus': -3.6,
             'personal_fouls': 178,
             'three_point_percentage': 0.368,
             'player_efficiency_rating': 23.0,
             'offensive_rebound_percentage': 2.4,
             'percentage_sixteen_foot_plus_two_pointers': 0.13,
             'three_point_attempt_rate': 0.364,
             'field_goals': 585,
             'free_throw_attempts': 792,
             'shooting_fouls_drawn': 329,
             'defensive_rebound_percentage': 11.9,
             'steal_percentage': 2.4,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.296,
             'percentage_ten_to_sixteen_footers': 0.049,
             'turnover_percentage': 14.9,
             'points_generated_by_assists': 1098,
             'block_percentage': 1.0,
             'field_goal_percentage': 0.438,
             'percentage_of_three_pointers_from_corner': 0.033,
             'free_throw_percentage': 0.851,
             'offensive_fouls': 37,
             'half_court_heaves': 3,
             'percentage_three_to_ten_footers': 0.149,
             'passing_turnovers': 143,
             'value_over_replacement_player': 5.7,
             'blocking_fouls': 0,
             'field_goal_attempts': 1337,
             'free_throw_attempt_rate': 0.592,
             'effective_field_goal_percentage': 0.504,
             'total_rebounds': 379,
             'center_percentage': 0,
             'power_forward_percentage': 0,
             'nationality': 'United States of America',
             'offensive_win_shares': 9.8,
             'defensive_rebounds': 317,
             'total_rebound_percentage': 7.2,
             'assists': 455,
             'defensive_box_plus_minus': 0.1,
             'shooting_distance': 13.7,
             'height': '6-5',
             'true_shooting_percentage': 0.6,
             'small_forward_percentage': 5,
             'minutes_played': 2985,
             'on_court_plus_minus': 2.8,
             'games_started': 78,
             'win_shares_per_48_minutes': 0.206,
             'dunks': 46,
             'three_pointers_assisted_percentage': 0.475,
             'two_point_attempts': 851,
             'salary': 5820417,
             'offensive_box_plus_minus': 5.4,
             'lost_ball_turnovers': 102,
             'percentage_shots_two_pointers': 0.636,
             'shooting_guard_percentage': 94,
             'shots_blocked': 106,
             'and_ones': 65,
             'three_pointers': 179,
             'take_fouls': 5,
             'free_throws': 674,
             'usage_percentage': 29.0,
             'assist_percentage': 25.7,
             'three_point_attempts': 486,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.364,
             'percentage_field_goals_as_dunks': 0.035,
             'offensive_rebounds': 62,
             'other_turnovers': 50,
             'point_guard_percentage': 0,
             'half_court_heaves_made': 1,
             'blocks': 38,
             'two_pointers_assisted_percentage': 0.241,
             'win_shares': 12.8,
             'defensive_win_shares': 3.0,
             'games_played': 78,
             'percentage_zero_to_three_footers': 0.308,
             'steals': 142,
             'turnovers': 295,
             'field_goal_perc_zero_to_three_feet': 0.65,
             'shooting_fouls': 67,
             'team_abbreviation': 'HOU',
             'points': 2023,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.25},
            {'field_goal_perc_ten_to_sixteen_feet': 0.437,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.368,
             'box_plus_minus': 10.9,
             'two_point_percentage': 0.531,
             'two_pointers': 386,
             'net_plus_minus': 5.3,
             'personal_fouls': 169,
             'three_point_percentage': 0.367,
             'player_efficiency_rating': 29.8,
             'offensive_rebound_percentage': 1.8,
             'percentage_sixteen_foot_plus_two_pointers': 0.039,
             'three_point_attempt_rate': 0.498,
             'field_goals': 651,
             'free_throw_attempts': 727,
             'shooting_fouls_drawn': 273,
             'defensive_rebound_percentage': 15.2,
             'steal_percentage': 2.4,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.351,
             'percentage_ten_to_sixteen_footers': 0.071,
             'turnover_percentage': 15.1,
             'points_generated_by_assists': 1499,
             'block_percentage': 1.7,
             'field_goal_percentage': 0.449,
             'percentage_of_three_pointers_from_corner': 0.072,
             'free_throw_percentage': 0.858,
             'offensive_fouls': 29,
             'half_court_heaves': 2,
             'percentage_three_to_ten_footers': 0.118,
             'passing_turnovers': 188,
             'value_over_replacement_player': 8.3,
             'blocking_fouls': 2,
             'field_goal_attempts': 1449,
             'free_throw_attempt_rate': 0.502,
             'effective_field_goal_percentage': 0.541,
             'total_rebounds': 389,
             'center_percentage': 0,
             'power_forward_percentage': 0,
             'nationality': 'United States of America',
             'offensive_win_shares': 11.6,
             'defensive_rebounds': 348,
             'total_rebound_percentage': 8.6,
             'assists': 630,
             'defensive_box_plus_minus': 1.3,
             'shooting_distance': 15.3,
             'height': '6-5',
             'true_shooting_percentage': 0.619,
             'small_forward_percentage': 6,
             'minutes_played': 2551,
             'on_court_plus_minus': 10.5,
             'games_started': 72,
             'win_shares_per_48_minutes': 0.289,
             'dunks': 19,
             'three_pointers_assisted_percentage': 0.26,
             'two_point_attempts': 727,
             'salary': 28299399,
             'offensive_box_plus_minus': 9.6,
             'lost_ball_turnovers': 87,
             'percentage_shots_two_pointers': 0.502,
             'shooting_guard_percentage': 75,
             'shots_blocked': 98,
             'and_ones': 55,
             'three_pointers': 265,
             'take_fouls': 5,
             'free_throws': 624,
             'usage_percentage': 36.1,
             'assist_percentage': 45.1,
             'three_point_attempts': 722,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.498,
             'percentage_field_goals_as_dunks': 0.013,
             'offensive_rebounds': 41,
             'other_turnovers': 40,
             'point_guard_percentage': 19,
             'half_court_heaves_made': 0,
             'blocks': 50,
             'two_pointers_assisted_percentage': 0.088,
             'win_shares': 15.4,
             'defensive_win_shares': 3.8,
             'games_played': 72,
             'percentage_zero_to_three_footers': 0.273,
             'steals': 126,
             'turnovers': 315,
             'field_goal_perc_zero_to_three_feet': 0.658,
             'shooting_fouls': 65,
             'team_abbreviation': 'HOU',
             'points': 2191,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.327},
            {'field_goal_perc_ten_to_sixteen_feet': 0.444,
             'field_goal_perc_sixteen_foot_plus_two_pointers': 0.402,
             'box_plus_minus': 2.3,
             'two_point_percentage': 0.514,
             'two_pointers': 185,
             'net_plus_minus': 2.7,
             'personal_fouls': 207,
             'three_point_percentage': 0.349,
             'player_efficiency_rating': 16.4,
             'offensive_rebound_percentage': 2.3,
             'percentage_sixteen_foot_plus_two_pointers': 0.127,
             'three_point_attempt_rate': 0.474,
             'field_goals': 298,
             'free_throw_attempts': 343,
             'shooting_fouls_drawn': 152,
             'defensive_rebound_percentage': 11.0,
             'steal_percentage': 2.2,
             'weight': 220,
             'field_goal_perc_three_to_ten_feet': 0.28,
             'percentage_ten_to_sixteen_footers': 0.026,
             'turnover_percentage': 11.3,
             'points_generated_by_assists': 419,
             'block_percentage': 0.8,
             'field_goal_percentage': 0.436,
             'percentage_of_three_pointers_from_corner': 0.182,
             'free_throw_percentage': 0.843,
             'offensive_fouls': 29,
             'half_court_heaves': 1,
             'percentage_three_to_ten_footers': 0.073,
             'passing_turnovers': 29,
             'value_over_replacement_player': 2.4,
             'blocking_fouls': 0,
             'field_goal_attempts': 684,
             'free_throw_attempt_rate': 0.501,
             'effective_field_goal_percentage': 0.518,
             'total_rebounds': 255,
             'center_percentage': 0,
             'power_forward_percentage': 1,
             'nationality': 'United States of America',
             'offensive_win_shares': 5.0,
             'defensive_rebounds': 213,
             'total_rebound_percentage': 6.8,
             'assists': 176,
             'defensive_box_plus_minus': -0.6,
             'shooting_distance': 15.4,
             'height': '6-5',
             'true_shooting_percentage': 0.598,
             'small_forward_percentage': 26,
             'minutes_played': 2189,
             'on_court_plus_minus': 5.5,
             'games_started': 5,
             'win_shares_per_48_minutes': 0.156,
             'dunks': 46,
             'three_pointers_assisted_percentage': 0.823,
             'two_point_attempts': 360,
             'salary': 4304520,
             'offensive_box_plus_minus': 2.9,
             'lost_ball_turnovers': 37,
             'percentage_shots_two_pointers': 0.526,
             'shooting_guard_percentage': 74,
             'shots_blocked': 46,
             'and_ones': 27,
             'three_pointers': 113,
             'take_fouls': 0,
             'free_throws': 289,
             'usage_percentage': 19.5,
             'assist_percentage': 12.8,
             'three_point_attempts': 324,
             'player_id': 'hardeja01',
             'percentage_shots_three_pointers': 0.474,
             'percentage_field_goals_as_dunks': 0.07,
             'offensive_rebounds': 42,
             'other_turnovers': 40,
             'point_guard_percentage': 0,
             'half_court_heaves_made': 0,
             'blocks': 24,
             'two_pointers_assisted_percentage': 0.422,
             'win_shares': 7.1,
             'defensive_win_shares': 2.1,
             'games_played': 82,
             'percentage_zero_to_three_footers': 0.3,
             'steals': 92,
             'turnovers': 106,
             'field_goal_perc_zero_to_three_feet': 0.624,
             'shooting_fouls': 81,
             'team_abbreviation': 'HOU',
             'points': 998,
             'position': 'SG',
             'three_point_shot_percentage_from_corner': 0.356}
        ]
        indices = ['2013-14', 'Career', '2014-15', '2016-17', '2009-10',
                   '2015-16', '2011-12', '2012-13', '2017-18', '2010-11']

        df = pd.DataFrame(dataframe, index=indices)
        player = self.player('')

        # Pandas doesn't natively allow comparisons of DataFrames.
        # Concatenating the two DataFrames (the one generated during the test
        # and the expected one above) and dropping duplicate rows leaves only
        # the rows that are unique between the two frames. This allows a quick
        # check of the DataFrame to see if it is empty - if so, all rows are
        # duplicates, and they are equal.
        frames = [df, player.dataframe]
        df1 = pd.concat(frames).drop_duplicates(keep=False)

    def test_nba_player_with_no_stats_handled_without_error(self):
        player = Player('youngtr01')

        assert player.name == 'Trae Young'


class TestNBARoster:
    @mock.patch('requests.get', side_effect=mock_pyquery)
    def test_roster_class_pulls_all_player_stats(self, *args, **kwargs):
        flexmock(utils) \
            .should_receive('_find_year_for_season') \
            .and_return('2018')
        roster = Roster('HOU')

        assert len(roster.players) == 4

        for player in roster.players:
            assert player.name in ['James Harden', 'Tarik Black',
                                   'Ryan Anderson', 'Trevor Ariza']

    @mock.patch('requests.get', side_effect=mock_pyquery)
    def test_bad_url_raises_value_error(self, *args, **kwargs):
        with pytest.raises(ValueError):
            roster = Roster('BAD')

    @mock.patch('requests.get', side_effect=mock_pyquery)
    def test_roster_from_team_class(self, *args, **kwargs):
        flexmock(Team) \
            .should_receive('_parse_team_data') \
            .and_return(None)
        team = Team(None, 1, '2018')
        mock_abbreviation = mock.PropertyMock(return_value='HOU')
        type(team)._abbreviation = mock_abbreviation

        assert len(team.roster.players) == 4

        for player in team.roster.players:
            assert player.name in ['James Harden', 'Tarik Black',
                                   'Ryan Anderson', 'Trevor Ariza']

        type(team)._abbreviation = None

    @mock.patch('requests.get', side_effect=mock_pyquery)
    def test_roster_class_with_slim_parameter(self, *args, **kwargs):
        flexmock(utils) \
            .should_receive('_find_year_for_season') \
            .and_return('2018')
        roster = Roster('HOU', slim=True)

        assert len(roster.players) == 4
        assert roster.players == {
            'hardeja01': 'James Harden',
            'blackta01': 'Tarik Black',
            'anderry01': 'Ryan Anderson',
            'arizatr01': 'Trevor Ariza'
        }



import mock
import os
import pandas as pd
import pytest
from datetime import datetime
from flexmock import flexmock
from sportsreference import utils
from sportsreference.constants import HOME, REGULAR_SEASON, WIN
from sportsreference.ncaaf.boxscore import Boxscore
from sportsreference.ncaaf.constants import SCHEDULE_URL
from sportsreference.ncaaf.schedule import Schedule


MONTH = 9
YEAR = 2017

NUM_GAMES_IN_SCHEDULE = 13


def read_file(filename):
    filepath = os.path.join(os.path.dirname(__file__), 'ncaaf', filename)
    return open('%s' % filepath, 'r').read()


def mock_pyquery(url):
    class MockPQ:
        def __init__(self, html_contents):
            self.status_code = 200
            self.html_contents = html_contents
            self.text = html_contents

        def __call__(self, div):
            return read_file('table.html' % YEAR)

    schedule = read_file('%s-schedule.html' % YEAR)
    return MockPQ(schedule)


class MockDateTime:
    def __init__(self, year, month):
        self.year = year
        self.month = month


class TestNCAAFSchedule:
    @mock.patch('requests.get', side_effect=mock_pyquery)
    def setup_method(self, *args, **kwargs):
        self.results = {
            'game': 2,
            'boxscore_index': '2017-09-09-michigan',
            'date': 'Sep 9, 2017',
            'time': '12:00 PM',
            'day_of_week': 'Sat',
            'datetime': datetime(2017, 9, 9, 12, 0),
            'location': HOME,
            'rank': 8,
            'opponent_abbr': 'cincinnati',
            'opponent_name': 'Cincinnati',
            'opponent_rank': None,
            'opponent_conference': 'American',
            'result': WIN,
            'points_for': 36,
            'points_against': 14,
            'wins': 2,
            'losses': 0,
            'streak': 'W 2'
        }
        flexmock(utils) \
            .should_receive('_todays_date') \
            .and_return(MockDateTime(YEAR, MONTH))
        flexmock(Boxscore) \
            .should_receive('_parse_game_data') \
            .and_return(None)
        flexmock(Boxscore) \
            .should_receive('dataframe') \
            .and_return(pd.DataFrame([{'key': 'value'}]))

        self.schedule = Schedule('MICHIGAN')

    def test_ncaaf_schedule_returns_correct_number_of_games(self):
        assert len(self.schedule) == NUM_GAMES_IN_SCHEDULE

    def test_ncaaf_schedule_returns_requested_match_from_index(self):
        match_two = self.schedule[1]

        for attribute, value in self.results.items():
            assert getattr(match_two, attribute) == value

    def test_ncaaf_schedule_returns_requested_match_from_date(self):
        match_two = self.schedule(datetime(2017, 9, 9))

        for attribute, value in self.results.items():
            assert getattr(match_two, attribute) == value

    def test_ncaaf_schedule_dataframe_returns_dataframe(self):
        df = pd.DataFrame([self.results], index=['MICHIGAN'])

        match_two = self.schedule[1]
        # Pandas doesn't natively allow comparisons of DataFrames.
        # Concatenating the two DataFrames (the one generated during the test
        # and the expected one above) and dropping duplicate rows leaves only
        # the rows that are unique between the two frames. This allows a quick
        # check of the DataFrame to see if it is empty - if so, all rows are
        # duplicates, and they are equal.
        frames = [df, match_two.dataframe]
        df1 = pd.concat(frames).drop_duplicates(keep=False)

        assert df1.empty

    def test_ncaaf_schedule_dataframe_extended_returns_dataframe(self):
        df = pd.DataFrame([{'key': 'value'}])

        result = self.schedule[1].dataframe_extended

        frames = [df, result]
        df1 = pd.concat(frames).drop_duplicates(keep=False)

        assert df1.empty

    def test_ncaaf_schedule_all_dataframe_returns_dataframe(self):
        result = self.schedule.dataframe.drop_duplicates(keep=False)

        assert len(result) == NUM_GAMES_IN_SCHEDULE
        assert set(result.columns.values) == set(self.results.keys())

    def test_ncaaf_schedule_all_dataframe_extended_returns_dataframe(self):
        result = self.schedule.dataframe_extended

        assert len(result) == NUM_GAMES_IN_SCHEDULE

    def test_no_games_for_date_raises_value_error(self):
        with pytest.raises(ValueError):
            self.schedule(datetime.now())
